# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import integer


class GitConfig(AWSProperty):
    props = {
        'Branch': (str, False),
        'RepositoryUrl': (str, True),
        'SecretArn': (str, False),
    }


class CodeRepository(AWSObject):
    resource_type = "AWS::SageMaker::CodeRepository"

    props = {
        'CodeRepositoryName': (str, False),
        'GitConfig': (GitConfig, True)
    }


class Endpoint(AWSObject):
    resource_type = "AWS::SageMaker::Endpoint"

    props = {
        'EndpointName': (str, False),
        'EndpointConfigName': (str, True),
        'Tags': (Tags, True)
    }


class ProductionVariant(AWSProperty):
    props = {
        'ModelName': (str, True),
        'VariantName': (str, True),
        'InitialInstanceCount': (integer, True),
        'InstanceType': (str, True),
        'InitialVariantWeight': (float, True)
    }


class EndpointConfig(AWSObject):
    resource_type = "AWS::SageMaker::EndpointConfig"

    props = {
        'EndpointConfigName': (str, False),
        'ProductionVariants': ([ProductionVariant], True),
        'KmsKeyId': (str, False),
        'Tags': (Tags, True)
    }


class ContainerDefinition(AWSProperty):
    props = {
        'ContainerHostname': (str, False),
        'Environment': (dict, False),
        'ModelDataUrl': (str, False),
        'Image': (str, True)
    }


class VpcConfig(AWSProperty):
    props = {
        'Subnets': ([str], True),
        'SecurityGroupIds': ([str], True)
    }


class Model(AWSObject):
    resource_type = "AWS::SageMaker::Model"

    props = {
        'ExecutionRoleArn': (str, True),
        'PrimaryContainer': (ContainerDefinition, True),
        'Containers': ([ContainerDefinition], False),
        'ModelName': (str, False),
        'VpcConfig': (VpcConfig, False),
        'Tags': (Tags, False)
    }


class NotebookInstanceLifecycleHook(AWSProperty):
    props = {
        'Content': (str, False)
    }


class NotebookInstanceLifecycleConfig(AWSObject):
    resource_type = "AWS::SageMaker::NotebookInstanceLifecycleConfig"

    props = {
        'NotebookInstanceLifecycleConfigName': (str, False),
        'OnCreate': ([NotebookInstanceLifecycleHook], False),
        'OnStart': ([NotebookInstanceLifecycleHook], False)
    }


class NotebookInstance(AWSObject):
    resource_type = "AWS::SageMaker::NotebookInstance"

    props = {
        'AcceleratorTypes': ([str], False),
        'AdditionalCodeRepositories': ([str], False),
        'DefaultCodeRepository': (str, False),
        'DirectInternetAccess': (str, False),
        'InstanceType': (str, True),
        'KmsKeyId': (str, False),
        'LifecycleConfigName': (str, False),
        'NotebookInstanceName': (str, False),
        'RoleArn': (str, True),
        'RootAccess': (str, False),
        'SecurityGroupIds': ([str], False),
        'SubnetId': (str, False),
        'Tags': (Tags, False),
        'VolumeSizeInGB': (integer, False),
    }


class CognitoMemberDefinition(AWSProperty):
    props = {
        'CognitoClientId': (str, True),
        'CognitoUserGroup': (str, True),
        'CognitoUserPool': (str, True),
    }


class MemberDefinition(AWSProperty):
    props = {
        'CognitoMemberDefinition': (CognitoMemberDefinition, True),
    }


class NotificationConfiguration(AWSProperty):
    props = {
        'NotificationTopicArn': (str, True),
    }


class Workteam(AWSObject):
    resource_type = "AWS::SageMaker::Workteam"

    props = {
        'Description': (str, False),
        'MemberDefinitions': ([MemberDefinition], False),
        'NotificationConfiguration': (NotificationConfiguration, False),
        'Tags': (Tags, False),
        'WorkteamName': (str, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.0.0


from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import integer
from .validators import double


class ADMChannel(AWSObject):
    resource_type = "AWS::Pinpoint::ADMChannel"

    props = {
        'ApplicationId': (str, True),
        'ClientId': (str, True),
        'ClientSecret': (str, True),
        'Enabled': (boolean, False),
    }


class APNSChannel(AWSObject):
    resource_type = "AWS::Pinpoint::APNSChannel"

    props = {
        'ApplicationId': (str, True),
        'BundleId': (str, False),
        'Certificate': (str, False),
        'DefaultAuthenticationMethod': (str, False),
        'Enabled': (boolean, False),
        'PrivateKey': (str, False),
        'TeamId': (str, False),
        'TokenKey': (str, False),
        'TokenKeyId': (str, False),
    }


class APNSSandboxChannel(AWSObject):
    resource_type = "AWS::Pinpoint::APNSSandboxChannel"

    props = {
        'ApplicationId': (str, True),
        'BundleId': (str, False),
        'Certificate': (str, False),
        'DefaultAuthenticationMethod': (str, False),
        'Enabled': (boolean, False),
        'PrivateKey': (str, False),
        'TeamId': (str, False),
        'TokenKey': (str, False),
        'TokenKeyId': (str, False),
    }


class APNSVoipChannel(AWSObject):
    resource_type = "AWS::Pinpoint::APNSVoipChannel"

    props = {
        'ApplicationId': (str, True),
        'BundleId': (str, False),
        'Certificate': (str, False),
        'DefaultAuthenticationMethod': (str, False),
        'Enabled': (boolean, False),
        'PrivateKey': (str, False),
        'TeamId': (str, False),
        'TokenKey': (str, False),
        'TokenKeyId': (str, False),
    }


class APNSVoipSandboxChannel(AWSObject):
    resource_type = "AWS::Pinpoint::APNSVoipSandboxChannel"

    props = {
        'ApplicationId': (str, True),
        'BundleId': (str, False),
        'Certificate': (str, False),
        'DefaultAuthenticationMethod': (str, False),
        'Enabled': (boolean, False),
        'PrivateKey': (str, False),
        'TeamId': (str, False),
        'TokenKey': (str, False),
        'TokenKeyId': (str, False),
    }


class App(AWSObject):
    resource_type = "AWS::Pinpoint::App"

    props = {
        'Name': (str, True),
        'Tags': (dict, False),
    }


class CampaignHook(AWSProperty):
    props = {
        'LambdaFunctionName': (str, False),
        'Mode': (str, False),
        'WebUrl': (str, False),
    }


class Limits(AWSProperty):
    props = {
        'Daily': (integer, False),
        'MaximumDuration': (integer, False),
        'MessagesPerSecond': (integer, False),
        'Total': (integer, False),
    }


class QuietTime(AWSProperty):
    props = {
        'End': (str, True),
        'Start': (str, True),
    }


class ApplicationSettings(AWSObject):
    resource_type = "AWS::Pinpoint::ApplicationSettings"

    props = {
        'ApplicationId': (str, True),
        'CampaignHook': (CampaignHook, False),
        'CloudWatchMetricsEnabled': (boolean, False),
        'Limits': (Limits, False),
        'QuietTime': (QuietTime, False),
    }


class BaiduChannel(AWSObject):
    resource_type = "AWS::Pinpoint::BaiduChannel"

    props = {
        'ApiKey': (str, True),
        'ApplicationId': (str, True),
        'Enabled': (boolean, False),
        'SecretKey': (str, True),
    }


class CampaignEmailMessage(AWSProperty):
    props = {
        'Body': (str, False),
        'FromAddress': (str, False),
        'HtmlBody': (str, False),
        'Title': (str, False),
    }


class CampaignSmsMessage(AWSProperty):
    props = {
        'Body': (str, False),
        'MessageType': (str, False),
        'SenderId': (str, False),
    }


class Message(AWSProperty):
    props = {
        'Action': (str, False),
        'Body': (str, False),
        'ImageIconUrl': (str, False),
        'ImageSmallIconUrl': (str, False),
        'ImageUrl': (str, False),
        'JsonBody': (str, False),
        'MediaUrl': (str, False),
        'RawContent': (str, False),
        'SilentPush': (boolean, False),
        'TimeToLive': (integer, False),
        'Title': (str, False),
        'Url': (str, False),
    }


class MessageConfiguration(AWSProperty):
    props = {
        'ADMMessage': (Message, False),
        'APNSMessage': (Message, False),
        'BaiduMessage': (Message, False),
        'DefaultMessage': (Message, False),
        'EmailMessage': (CampaignEmailMessage, False),
        'GCMMessage': (Message, False),
        'SMSMessage': (CampaignSmsMessage, False),
    }


class SetDimension(AWSProperty):
    props = {
        'DimensionType': (str, False),
        'Values': ([str], False),
    }


class EventDimensions(AWSProperty):
    props = {
        'Attributes': (dict, False),
        'EventType': (SetDimension, False),
        'Metrics': (dict, False),
    }


class CampaignEventFilter(AWSProperty):
    props = {
        'Dimensions': (EventDimensions, False),
        'FilterType': (str, False),
    }


class Schedule(AWSProperty):
    props = {
        'EndTime': (str, False),
        'EventFilter': (CampaignEventFilter, False),
        'Frequency': (str, False),
        'IsLocalTime': (boolean, False),
        'QuietTime': (QuietTime, False),
        'StartTime': (str, False),
        'TimeZone': (str, False),
    }


class WriteTreatmentResource(AWSProperty):
    props = {
        'MessageConfiguration': (MessageConfiguration, False),
        'Schedule': (Schedule, False),
        'SizePercent': (integer, False),
        'TreatmentDescription': (str, False),
        'TreatmentName': (str, False),
    }


class Campaign(AWSObject):
    resource_type = "AWS::Pinpoint::Campaign"

    props = {
        'AdditionalTreatments': ([WriteTreatmentResource], False),
        'ApplicationId': (str, True),
        'CampaignHook': (CampaignHook, False),
        'Description': (str, False),
        'HoldoutPercent': (integer, False),
        'IsPaused': (boolean, False),
        'Limits': (Limits, False),
        'MessageConfiguration': (MessageConfiguration, True),
        'Name': (str, True),
        'Schedule': (Schedule, True),
        'SegmentId': (str, True),
        'SegmentVersion': (integer, False),
        'Tags': (dict, False),
        'TreatmentDescription': (str, False),
        'TreatmentName': (str, False),
    }


class EmailChannel(AWSObject):
    resource_type = "AWS::Pinpoint::EmailChannel"

    props = {
        'ApplicationId': (str, True),
        'ConfigurationSet': (str, False),
        'Enabled': (boolean, False),
        'FromAddress': (str, True),
        'Identity': (str, True),
        'RoleArn': (str, False),
    }


class EmailTemplate(AWSObject):
    resource_type = "AWS::Pinpoint::EmailTemplate"

    props = {
        'DefaultSubstitutions': (str, False),
        'HtmlPart': (str, False),
        'Subject': (str, True),
        'Tags': (dict, False),
        'TemplateDescription': (str, False),
        'TemplateName': (str, True),
        'TextPart': (str, False),
    }


class EventStream(AWSObject):
    resource_type = "AWS::Pinpoint::EventStream"

    props = {
        'ApplicationId': (str, True),
        'DestinationStreamArn': (str, True),
        'RoleArn': (str, True),
    }


class GCMChannel(AWSObject):
    resource_type = "AWS::Pinpoint::GCMChannel"

    props = {
        'ApiKey': (str, True),
        'ApplicationId': (str, True),
        'Enabled': (boolean, False),
    }


class APNSPushNotificationTemplate(AWSProperty):
    props = {
        'Action': (str, False),
        'Body': (str, False),
        'MediaUrl': (str, False),
        'Sound': (str, False),
        'Title': (str, False),
        'Url': (str, False),
    }


class AndroidPushNotificationTemplate(AWSProperty):
    props = {
        'Action': (str, False),
        'Body': (str, False),
        'ImageIconUrl': (str, False),
        'ImageUrl': (str, False),
        'SmallImageIconUrl': (str, False),
        'Sound': (str, False),
        'Title': (str, False),
        'Url': (str, False),
    }


class DefaultPushNotificationTemplate(AWSProperty):
    props = {
        'Action': (str, False),
        'Body': (str, False),
        'Sound': (str, False),
        'Title': (str, False),
        'Url': (str, False),
    }


class PushTemplate(AWSObject):
    resource_type = "AWS::Pinpoint::PushTemplate"

    props = {
        'ADM': (AndroidPushNotificationTemplate, False),
        'APNS': (APNSPushNotificationTemplate, False),
        'Baidu': (AndroidPushNotificationTemplate, False),
        'Default': (DefaultPushNotificationTemplate, False),
        'DefaultSubstitutions': (str, False),
        'GCM': (AndroidPushNotificationTemplate, False),
        'Tags': (dict, False),
        'TemplateDescription': (str, False),
        'TemplateName': (str, True),
    }


class SMSChannel(AWSObject):
    resource_type = "AWS::Pinpoint::SMSChannel"

    props = {
        'ApplicationId': (str, True),
        'Enabled': (boolean, False),
        'SenderId': (str, False),
        'ShortCode': (str, False),
    }


class Recency(AWSProperty):
    props = {
        'Duration': (str, True),
        'RecencyType': (str, True),
    }


class Behavior(AWSProperty):
    props = {
        'Recency': (Recency, False),
    }


class Demographic(AWSProperty):
    props = {
        'AppVersion': (SetDimension, False),
        'Channel': (SetDimension, False),
        'DeviceType': (SetDimension, False),
        'Make': (SetDimension, False),
        'Model': (SetDimension, False),
        'Platform': (SetDimension, False),
    }


class Coordinates(AWSProperty):
    props = {
        'Latitude': (double, True),
        'Longitude': (double, True),
    }


class GPSPoint(AWSProperty):
    props = {
        'Coordinates': (Coordinates, True),
        'RangeInKilometers': (double, True),
    }


class Location(AWSProperty):
    props = {
        'Country': (SetDimension, False),
        'GPSPoint': (GPSPoint, False),
    }


class SegmentDimensions(AWSProperty):
    props = {
        'Attributes': (dict, False),
        'Behavior': (Behavior, False),
        'Demographic': (Demographic, False),
        'Location': (Location, False),
        'Metrics': (dict, False),
        'UserAttributes': (dict, False),
    }


class SourceSegments(AWSProperty):
    props = {
        'Id': (str, True),
        'Version': (integer, False),
    }


class Groups(AWSProperty):
    props = {
        'Dimensions': ([SegmentDimensions], False),
        'SourceSegments': ([SourceSegments], False),
        'SourceType': (str, False),
        'Type': (str, False),
    }


class SegmentGroups(AWSProperty):
    props = {
        'Groups': ([Groups], False),
        'Include': (str, False),
    }


class Segment(AWSObject):
    resource_type = "AWS::Pinpoint::Segment"

    props = {
        'ApplicationId': (str, True),
        'Dimensions': (SegmentDimensions, False),
        'Name': (str, True),
        'SegmentGroups': (SegmentGroups, False),
        'Tags': (dict, False),
    }


class SmsTemplate(AWSObject):
    resource_type = "AWS::Pinpoint::SmsTemplate"

    props = {
        'Body': (str, True),
        'DefaultSubstitutions': (str, False),
        'Tags': (dict, False),
        'TemplateDescription': (str, False),
        'TemplateName': (str, True),
    }


class VoiceChannel(AWSObject):
    resource_type = "AWS::Pinpoint::VoiceChannel"

    props = {
        'ApplicationId': (str, True),
        'Enabled': (boolean, False),
    }



#
# Regions
#

AP_EAST_1 = 'ap-east-1'
AP_NORTHEAST_1 = 'ap-northeast-1'
AP_NORTHEAST_2 = 'ap-northeast-2'
AP_NORTHEAST_3 = 'ap-northeast-3'
AP_SOUTHEAST_1 = 'ap-southeast-1'
AP_SOUTHEAST_2 = 'ap-southeast-2'
AP_SOUTH_1 = 'ap-south-1'
CA_CENTRAL_1 = 'ca-central-1'
CN_NORTH_1 = 'cn-north-1'
CN_NORTHWEST_1 = 'cn-northwest-1'
EU_WEST_1 = 'eu-west-1'
EU_WEST_2 = 'eu-west-2'
EU_WEST_3 = 'eu-west-3'
EU_CENTRAL_1 = 'eu-central-1'
EU_NORTH_1 = 'eu-north-1'
SA_EAST_1 = 'sa-east-1'
US_EAST_1 = 'us-east-1'
US_EAST_2 = 'us-east-2'
US_GOV_EAST_1 = 'us-gov-east-1'
US_GOV_WEST_1 = 'us-gov-west-1'
US_WEST_1 = 'us-west-1'
US_WEST_2 = 'us-west-2'

#
# Availability Zones
#

AP_NORTHEAST_1A = 'ap-northeast-1a'
AP_NORTHEAST_1B = 'ap-northeast-1b'
AP_NORTHEAST_1C = 'ap-northeast-1c'

AP_NORTHEAST_2A = 'ap-northeast-2a'
AP_NORTHEAST_2B = 'ap-northeast-2b'
AP_NORTHEAST_2C = 'ap-northeast-2c'

AP_NORTHEAST_3A = 'ap-northeast-3a'

AP_SOUTHEAST_1A = 'ap-southeast-1a'
AP_SOUTHEAST_1B = 'ap-southeast-1b'
AP_SOUTHEAST_1C = 'ap-southeast-1c'

AP_SOUTHEAST_2A = 'ap-southeast-2a'
AP_SOUTHEAST_2B = 'ap-southeast-2b'
AP_SOUTHEAST_2C = 'ap-southeast-2c'

AP_SOUTH_1A = 'ap-south-1a'
AP_SOUTH_1B = 'ap-south-1b'

CA_CENTRAL_1A = 'ca-central-1a'
CA_CENTRAL_1B = 'ca-central-1b'

CN_NORTH_1A = 'cn-north-1a'
CN_NORTH_1B = 'cn-north-1b'

CN_NORTHWEST_1A = 'cn-northwest-1a'
CN_NORTHWEST_1B = 'cn-northwest-1b'
CN_NORTHWEST_1C = 'cn-northwest-1c'

EU_WEST_1A = 'eu-west-1a'
EU_WEST_1B = 'eu-west-1b'
EU_WEST_1C = 'eu-west-1c'

EU_WEST_2A = 'eu-west-2a'
EU_WEST_2B = 'eu-west-2b'
EU_WEST_2C = 'eu-west-2c'

EU_WEST_3A = 'eu-west-3a'
EU_WEST_3B = 'eu-west-3b'
EU_WEST_3C = 'eu-west-3c'

EU_CENTRAL_1A = 'eu-central-1a'
EU_CENTRAL_1B = 'eu-central-1b'
EU_CENTRAL_1C = 'eu-central-1c'

EU_NORTH_1A = 'eu-north-1a'
EU_NORTH_1B = 'eu-north-1b'
EU_NORTH_1C = 'eu-north-1c'

SA_EAST_1A = 'sa-east-1a'
SA_EAST_1B = 'sa-east-1b'
SA_EAST_1C = 'sa-east-1c'

US_EAST_1A = 'us-east-1a'
US_EAST_1B = 'us-east-1b'
US_EAST_1C = 'us-east-1c'
US_EAST_1D = 'us-east-1d'
US_EAST_1E = 'us-east-1e'
US_EAST_1F = 'us-east-1f'

US_EAST_2A = 'us-east-2a'
US_EAST_2B = 'us-east-2b'
US_EAST_2C = 'us-east-2c'

US_WEST_1A = 'us-west-1a'
US_WEST_1B = 'us-west-1b'
US_WEST_1C = 'us-west-1c'

US_GOV_EAST_1A = 'us-gov-east-1a'
US_GOV_EAST_1B = 'us-gov-east-1b'
US_GOV_EAST_1C = 'us-gov-east-1c'

US_GOV_WEST_1A = 'us-gov-west-1a'
US_GOV_WEST_1B = 'us-gov-west-1b'
US_GOV_WEST_1C = 'us-gov-west-1c'

US_WEST_2A = 'us-west-2a'
US_WEST_2B = 'us-west-2b'
US_WEST_2C = 'us-west-2c'
US_WEST_2D = 'us-west-2d'

#
# Networking
#

QUAD_ZERO = '0.0.0.0/0'
VPC_CIDR_16 = '10.0.0.0/16'

SSH_PORT = 22
MONGODB_PORT = 27017
NTP_PORT = 123
SMTP_PORT_25 = 25
SMTP_PORT_587 = 587
HTTP_PORT = 80
HTTPS_PORT = 443
REDIS_PORT = 6379
MEMCACHED_PORT = 11211
POSTGRESQL_PORT = 5432

TCP_PROTOCOL = 6
UDP_PROTOCOL = 17
ICMP_PROTOCOL = 1
ALL_PROTOCOL = -1

#
# EC2 instance types
#

T2_NANO = 't2.nano'
T2_MICRO = 't2.micro'
T2_SMALL = 't2.small'
T2_MEDIUM = 't2.medium'
T2_LARGE = 't2.large'
T2_XLARGE = 't2.xlarge'
T2_2XLARGE = 't2.2xlarge'

T3_NANO = 't3.nano'
T3_MICRO = 't3.micro'
T3_SMALL = 't3.small'
T3_MEDIUM = 't3.medium'
T3_LARGE = 't3.large'
T3_XLARGE = 't3.xlarge'
T3_2XLARGE = 't3.2xlarge'

T3A_NANO = 't3a.nano'
T3A_MICRO = 't3a.micro'
T3A_SMALL = 't3a.small'
T3A_MEDIUM = 't3a.medium'
T3A_LARGE = 't3a.large'
T3A_XLARGE = 't3a.xlarge'
T3A_2XLARGE = 't3a.2xlarge'

M5_LARGE = 'm5.large'
M5_XLARGE = 'm5.xlarge'
M5_2XLARGE = 'm5.2xlarge'
M5_4XLARGE = 'm5.4xlarge'
M5_12XLARGE = 'm5.12xlarge'
M5_24XLARGE = 'm5.24xlarge'
M5_METAL = 'm5.metal'

M5D_LARGE = 'm5d.large'
M5D_XLARGE = 'm5d.xlarge'
M5D_2XLARGE = 'm5d.2xlarge'
M5D_4XLARGE = 'm5d.4xlarge'
M5D_12XLARGE = 'm5d.12xlarge'
M5D_24XLARGE = 'm5d.24xlarge'
M5D_METAL = 'm5d.metal'

M5A_LARGE = 'm5a.large'
M5A_XLARGE = 'm5a.xlarge'
M5A_2XLARGE = 'm5a.2xlarge'
M5A_4XLARGE = 'm5a.4xlarge'
M5A_12XLARGE = 'm5a.12xlarge'
M5A_24XLARGE = 'm5a.24xlarge'

M5AD_LARGE = 'm5ad.large'
M5AD_XLARGE = 'm5ad.xlarge'
M5AD_2XLARGE = 'm5ad.2xlarge'
M5AD_4XLARGE = 'm5ad.4xlarge'
M5AD_12XLARGE = 'm5ad.12xlarge'
M5AD_24XLARGE = 'm5ad.24xlarge'

M4_LARGE = 'm4.large'
M4_XLARGE = 'm4.xlarge'
M4_2XLARGE = 'm4.2xlarge'
M4_4XLARGE = 'm4.4xlarge'
M4_10XLARGE = 'm4.10xlarge'
M4_16XLARGE = 'm4.16xlarge'

M3_MEDIUM = 'm3.medium'
M3_LARGE = 'm3.large'
M3_XLARGE = 'm3.xlarge'
M3_2XLARGE = 'm3.2xlarge'

A1_MEDIUM = 'a1.medium'
A1_LARGE = 'a1.large'
A1_XLARGE = 'a1.xlarge'
A1_2XLARGE = 'a1.2xlarge'
A1_4XLARGE = 'a1.4xlarge'

C3_LARGE = 'c3.large'
C3_XLARGE = 'c3.xlarge'
C3_2XLARGE = 'c3.2xlarge'
C3_4XLARGE = 'c3.4xlarge'
C3_8XLARGE = 'c3.8xlarge'

C4_LARGE = 'c4.large'
C4_XLARGE = 'c4.xlarge'
C4_2XLARGE = 'c4.2xlarge'
C4_4XLARGE = 'c4.4xlarge'
C4_8XLARGE = 'c4.8xlarge'

C5_LARGE = 'c5.large'
C5_XLARGE = 'c5.xlarge'
C5_2XLARGE = 'c5.2xlarge'
C5_4XLARGE = 'c5.4xlarge'
C5_9XLARGE = 'c5.9xlarge'
C5_18XLARGE = 'c5.18xlarge'

C5D_LARGE = 'c5d.large'
C5D_XLARGE = 'c5d.xlarge'
C5D_2XLARGE = 'c5d.2xlarge'
C5D_4XLARGE = 'c5d.4xlarge'
C5D_9XLARGE = 'c5d.9xlarge'
C5D_18XLARGE = 'c5d.18xlarge'

C5N_LARGE = 'c5n.large'
C5N_XLARGE = 'c5n.xlarge'
C5N_2XLARGE = 'c5n.2xlarge'
C5N_4XLARGE = 'c5n.4xlarge'
C5N_9XLARGE = 'c5n.9xlarge'
C5N_18XLARGE = 'c5n.18xlarge'

R3_LARGE = 'r3.large'
R3_XLARGE = 'r3.xlarge'
R3_2XLARGE = 'r3.2xlarge'
R3_4XLARGE = 'r3.4xlarge'
R3_8XLARGE = 'r3.8xlarge'

G2_2XLARGE = 'g2.2xlarge'
G2_8XLARGE = 'g2.8xlarge'

G3S_XLARGE = 'g3s.xlarge'
G3_4XLARGE = 'g3.4xlarge'
G3_8XLARGE = 'g3.8xlarge'
G3_16XLARGE = 'g3.16xlarge'

I2_XLARGE = 'i2.xlarge'
I2_2XLARGE = 'i2.2xlarge'
I2_4XLARGE = 'i2.4xlarge'
I2_8XLARGE = 'i2.8xlarge'

H1_2XLARGE = 'h1.2xlarge'
H1_4XLARGE = 'h1.4xlarge'
H1_8XLARGE = 'h1.8xlarge'
H1_16XLARGE = 'h1.16xlarge'

I3_LARGE = 'i3.large'
I3_XLARGE = 'i3.xlarge'
I3_2XLARGE = 'i3.2xlarge'
I3_4XLARGE = 'i3.4xlarge'
I3_8XLARGE = 'i3.8xlarge'
I3_16XLARGE = 'i3.16xlarge'
I3_METAL = 'i3.metal'

I3EN_LARGE = 'i3en.large'
I3EN_XLARGE = 'i3en.xlarge'
I3EN_2XLARGE = 'i3en.2xlarge'
I3EN_3XLARGE = 'i3en.3xlarge'
I3EN_6XLARGE = 'i3en.6xlarge'
I3EN_12XLARGE = 'i3en.12xlarge'
I3EN_24XLARGE = 'i3en.24xlarge'

D2_XLARGE = 'd2.xlarge'
D2_2XLARGE = 'd2.2xlarge'
D2_4XLARGE = 'd2.4xlarge'
D2_8XLARGE = 'd2.8xlarge'

HS1_8XLARGE = 'hs1.8xlarge'

M1_SMALL = 'm1.small'
M1_MEDIUM = 'm1.medium'
M1_LARGE = 'm1.large'
M1_XLARGE = 'm1.xlarge'

C1_MEDIUM = 'c1.medium'
C1_XLARGE = 'c1.xlarge'
CC2_8XLARGE = 'cc2.8xlarge'

CG1_4XLARGE = 'cg1.4xlarge'

M2_XLARGE = 'm2.xlarge'
M2_2XLARGE = 'm2.2xlarge'
M2_4XLARGE = 'm2.4xlarge'
CR1_8XLARGE = 'cr1.8xlarge'

HI1_4XLARGE = 'hi1.4xlarge'

T1_MICRO = 't1.micro'

X1_32XLARGE = 'x1.32xlarge'
X1_16XLARGE = 'x1.16xlarge'

X1E_XLARGE = 'x1e.xlarge'
X1E_2XLARGE = 'x1e.2xlarge'
X1E_4XLARGE = 'x1e.4xlarge'
X1E_8XLARGE = 'x1e.8xlarge'
X1E_16XLARGE = 'x1e.16xlarge'
X1E_32XLARGE = 'x1e.32xlarge'

U_6TB1_METAL = 'u-6tb1.metal'
U_9TB1_METAL = 'u-9tb1.metal'
U_12TB1_METAL = 'u-12tb1.metal'

Z1D_LARGE = 'z1d.large'
Z1D_XLARGE = 'z1d.xlarge'
Z1D_2XLARGE = 'z1d.2xlarge'
Z1D_3XLARGE = 'z1d.3xlarge'
Z1D_6XLARGE = 'z1d.6xlarge'
Z1D_12XLARGE = 'z1d.12xlarge'
Z1D_METAL = 'z1d.metal'

R4_LARGE = 'r4.large'
R4_XLARGE = 'r4.xlarge'
R4_2XLARGE = 'r4.2xlarge'
R4_4XLARGE = 'r4.4xlarge'
R4_8XLARGE = 'r4.8xlarge'
R4_16XLARGE = 'r4.16xlarge'

R5_LARGE = 'r5.large'
R5_XLARGE = 'r5.xlarge'
R5_2XLARGE = 'r5.2xlarge'
R5_4XLARGE = 'r5.4xlarge'
R5_12XLARGE = 'r5.12xlarge'
R5_24XLARGE = 'r5.24xlarge'
R5_METAL = 'r5.metal'

R5D_LARGE = 'r5d.large'
R5D_XLARGE = 'r5d.xlarge'
R5D_2XLARGE = 'r5d.2xlarge'
R5D_4XLARGE = 'r5d.4xlarge'
R5D_12XLARGE = 'r5d.12xlarge'
R5D_24XLARGE = 'r5d.24xlarge'
R5D_METAL = 'r5d.metal'

R5A_LARGE = 'r5a.large'
R5A_XLARGE = 'r5a.xlarge'
R5A_2XLARGE = 'r5a.2xlarge'
R5A_4XLARGE = 'r5a.4xlarge'
R5A_12XLARGE = 'r5a.12xlarge'
R5A_24XLARGE = 'r5a.24xlarge'

R5AD_LARGE = 'r5ad.large'
R5AD_XLARGE = 'r5ad.xlarge'
R5AD_2XLARGE = 'r5ad.2xlarge'
R5AD_4XLARGE = 'r5ad.4xlarge'
R5AD_12XLARGE = 'r5ad.12xlarge'
R5AD_24XLARGE = 'r5ad.24xlarge'

P2_XLARGE = 'p2.xlarge'
P2_8XLARGE = 'p2.8xlarge'
P2_16XLARGE = 'p2.16xlarge'

P3_2XLARGE = 'p3.2xlarge'
P3_8XLARGE = 'p3.8xlarge'
P3_16XLARGE = 'p3.16xlarge'
P3DN_24XLARGE = 'p3dn.24xlarge'

F1_2XLARGE = 'f1.2xlarge'
F1_4XLARGE = 'f1.4xlarge'
F1_16XLARGE = 'f1.16xlarge'

#
# RDS DB instance classes
#

DB_T1_MICRO = 'db.t1.micro'

DB_T2_MICRO = 'db.t2.micro'
DB_T2_SMALL = 'db.t2.small'
DB_T2_MEDIUM = 'db.t2.medium'
DB_T2_LARGE = 'db.t2.large'
DB_T2_XLARGE = 'db.t2.xlarge'
DB_T2_2XLARGE = 'db.t2.2xlarge'

DB_T3_MICRO = 'db.t3.micro'
DB_T3_SMALL = 'db.t3.small'
DB_T3_MEDIUM = 'db.t3.medium'
DB_T3_LARGE = 'db.t3.large'
DB_T3_XLARGE = 'db.t3.xlarge'
DB_T3_2XLARGE = 'db.t3.2xlarge'

DB_M1_SMALL = 'db.m1.small'
DB_M1_MEDIUM = 'db.m1.medium'
DB_M1_LARGE = 'db.m1.large'
DB_M1_XLARGE = 'db.m1.xlarge'

DB_M2_XLARGE = 'db.m2.xlarge'
DB_M2_2XLARGE = 'db.m2.2xlarge'
DB_M2_4XLARGE = 'db.m2.4xlarge'

DB_M3_MEDIUM = 'db.m3.medium'
DB_M3_LARGE = 'db.m3.large'
DB_M3_XLARGE = 'db.m3.xlarge'
DB_M3_2XLARGE = 'db.m3.2xlarge'

DB_M4_LARGE = 'db.m4.large'
DB_M4_XLARGE = 'db.m4.xlarge'
DB_M4_2XLARGE = 'db.m4.2xlarge'
DB_M4_4XLARGE = 'db.m4.4xlarge'
DB_M4_10XLARGE = 'db.m4.10xlarge'
DB_M4_16XLARGE = 'db.m4.16xlarge'

DB_M5_LARGE = 'db.m5.large'
DB_M5_XLARGE = 'db.m5.xlarge'
DB_M5_2XLARGE = 'db.m5.2xlarge'
DB_M5_4XLARGE = 'db.m5.4xlarge'
DB_M5_12XLARGE = 'db.m5.12xlarge'
DB_M5_24XLARGE = 'db.m5.24xlarge'

DB_R3_LARGE = 'db.r3.large'
DB_R3_XLARGE = 'db.r3.xlarge'
DB_R3_2XLARGE = 'db.r3.2xlarge'
DB_R3_4XLARGE = 'db.r3.4xlarge'
DB_R3_8XLARGE = 'db.r3.8xlarge'

DB_R4_LARGE = 'db.r4.large'
DB_R4_XLARGE = 'db.r4.xlarge'
DB_R4_2XLARGE = 'db.r4.2xlarge'
DB_R4_4XLARGE = 'db.r4.4xlarge'
DB_R4_8XLARGE = 'db.r4.8xlarge'
DB_R4_16XLARGE = 'db.r4.16xlarge'

DB_R5_LARGE = 'db.r5.large'
DB_R5_XLARGE = 'db.r5.xlarge'
DB_R5_2XLARGE = 'db.r5.2xlarge'
DB_R5_4XLARGE = 'db.r5.4xlarge'
DB_R5_12XLARGE = 'db.r5.12xlarge'
DB_R5_24XLARGE = 'db.r5.24xlarge'

DB_X1_16XLARGE = 'db.x1.16xlarge'
DB_X1_32XLARGE = 'db.x1.32xlarge'

DB_X1E_XLARGE = 'db.x1e.xlarge'
DB_X1E_2XLARGE = 'db.x1e.2xlarge'
DB_X1E_4XLARGE = 'db.x1e.4xlarge'
DB_X1E_8XLARGE = 'db.x1e.8xlarge'
DB_X1E_16XLARGE = 'db.x1e.16xlarge'
DB_X1E_32XLARGE = 'db.x1e.32xlarge'

DB_CR1_8XLARGE = 'db.cr1.8xlarge'

#
# ElastiCache node types
#

CACHE_T2_MICRO = 'cache.t2.micro'
CACHE_T2_SMALL = 'cache.t2.small'
CACHE_T2_MEDIUM = 'cache.t2.medium'

CACHE_M3_MEDIUM = 'cache.m3.medium'
CACHE_M3_LARGE = 'cache.m3.large'
CACHE_M3_XLARGE = 'cache.m3.xlarge'
CACHE_M3_2XLARGE = 'cache.m3.2xlarge'

CACHE_R3_LARGE = 'cache.r3.large'
CACHE_R3_XLARGE = 'cache.r3.xlarge'
CACHE_R3_2XLARGE = 'cache.r3.2xlarge'
CACHE_R3_4XLARGE = 'cache.r3.4xlarge'
CACHE_R3_8XLARGE = 'cache.r3.8xlarge'

CACHE_M1_SMALL = 'cache.m1.small'
CACHE_M1_MEDIUM = 'cache.m1.medium'
CACHE_M1_LARGE = 'cache.m1.large'
CACHE_M1_XLARGE = 'cache.m1.xlarge'

CACHE_M2_XLARGE = 'cache.m2.xlarge'
CACHE_M2_2XLARGE = 'cache.m2.2xlarge'
CACHE_M2_4XLARGE = 'cache.m2.4xlarge'

CACHE_C1_XLARGE = 'cache.c1.xlarge'

CACHE_T1_MICRO = 'cache.t1.micro'

#
# Elasticsearch instance types
#

ELASTICSEARCH_T2_MICRO = 't2.micro.elasticsearch'
ELASTICSEARCH_T2_SMALL = 't2.small.elasticsearch'
ELASTICSEARCH_T2_MEDIUM = 't2.medium.elasticsearch'

ELASTICSEARCH_M3_MEDIUM = 'm3.medium.elasticsearch'
ELASTICSEARCH_M3_LARGE = 'm3.large.elasticsearch'
ELASTICSEARCH_M3_XLARGE = 'm3.xlarge.elasticsearch'
ELASTICSEARCH_M3_2XLARGE = 'm3.2xlarge.elasticsearch'

ELASTICSEARCH_M4_LARGE = 'm4.large.elasticsearch'
ELASTICSEARCH_M4_XLARGE = 'm4.xlarge.elasticsearch'
ELASTICSEARCH_M4_2XLARGE = 'm4.2xlarge.elasticsearch'
ELASTICSEARCH_M4_4XLARGE = 'm4.4xlarge.elasticsearch'
ELASTICSEARCH_M4_10XLARGE = 'm4.10xlarge.elasticsearch'

ELASTICSEARCH_C4_LARGE = 'c4.large.elasticsearch'
ELASTICSEARCH_C4_XLARGE = 'c4.xlarge.elasticsearch'
ELASTICSEARCH_C4_2XLARGE = 'c4.2xlarge.elasticsearch'
ELASTICSEARCH_C4_4XLARGE = 'c4.4xlarge.elasticsearch'
ELASTICSEARCH_C4_8XLARGE = 'c4.8xlarge.elasticsearch'

ELASTICSEARCH_R3_LARGE = 'r3.large.elasticsearch'
ELASTICSEARCH_R3_XLARGE = 'r3.xlarge.elasticsearch'
ELASTICSEARCH_R3_2XLARGE = 'r3.2xlarge.elasticsearch'
ELASTICSEARCH_R3_4XLARGE = 'r3.4xlarge.elasticsearch'
ELASTICSEARCH_R3_8XLARGE = 'r3.8xlarge.elasticsearch'

ELASTICSEARCH_R4_LARGE = 'r4.large.elasticsearch'
ELASTICSEARCH_R4_XLARGE = 'r4.xlarge.elasticsearch'
ELASTICSEARCH_R4_2XLARGE = 'r4.2xlarge.elasticsearch'
ELASTICSEARCH_R4_4XLARGE = 'r4.4xlarge.elasticsearch'
ELASTICSEARCH_R4_8XLARGE = 'r4.8xlarge.elasticsearch'
ELASTICSEARCH_R4_16XLARGE = 'r4.16xlarge.elasticsearch'

ELASTICSEARCH_I2_XLARGE = 'i2.xlarge.elasticsearch'
ELASTICSEARCH_I2_2XLARGE = 'i2.2xlarge.elasticsearch'

#
# Kafka brokers instance types
#

KAFKA_M5_LARGE = 'kafka.m5.large'
KAFKA_M5_XLARGE = 'kafka.m5.xlarge'
KAFKA_M5_2XLARGE = 'kafka.m5.2xlarge'
KAFKA_M5_4XLARGE = 'kafka.m5.4xlarge'
KAFKA_M5_12XLARGE = 'kafka.m5.12xlarge'
KAFKA_M5_24XLARGE = 'kafka.m5.24xlarge'

#
# Parameter types
#

STRING = 'String'
NUMBER = 'Number'
LIST_OF_NUMBERS = 'List<Number>'
COMMA_DELIMITED_LIST = 'CommaDelimitedList'

AVAILABILITY_ZONE_NAME = 'AWS::EC2::AvailabilityZone::Name'
IMAGE_ID = 'AWS::EC2::Image::Id'
INSTANCE_ID = 'AWS::EC2::Instance::Id'
KEY_PAIR_NAME = 'AWS::EC2::KeyPair::KeyName'
SECURITY_GROUP_NAME = 'AWS::EC2::SecurityGroup::GroupName'
SECURITY_GROUP_ID = 'AWS::EC2::SecurityGroup::Id'
SUBNET_ID = 'AWS::EC2::Subnet::Id'
VOLUME_ID = 'AWS::EC2::Volume::Id'
VPC_ID = 'AWS::EC2::VPC::Id'
HOSTED_ZONE_ID = 'AWS::Route53::HostedZone::Id'

LIST_OF_AVAILABILITY_ZONE_NAMES = 'List<AWS::EC2::AvailabilityZone::Name>'
LIST_OF_IMAGE_ID = 'List<AWS::EC2::Image::Id>'
LIST_OF_INSTANCE_IDS = 'List<AWS::EC2::Instance::Id>'
LIST_OF_SECURITY_GROUP_NAMES = 'List<AWS::EC2::SecurityGroup::GroupName>'
LIST_OF_SECURITY_GROUP_IDS = 'List<AWS::EC2::SecurityGroup::Id>'
LIST_OF_SUBNET_IDS = 'List<AWS::EC2::Subnet::Id>'
LIST_OF_VOLUME_IDS = 'List<AWS::EC2::Volume::Id>'
LIST_OF_VPC_IDS = 'List<AWS::EC2::VPC::Id>'
LIST_OF_HOSTED_ZONE_IDS = 'List<AWS::Route53::HostedZone::Id>'

#
# Logs
#
LOGS_ALLOWED_RETENTION_DAYS = [1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180,
                               365, 400, 545, 731, 1827, 3653]

#
# Route53
#

CLOUDFRONT_HOSTEDZONEID = 'Z2FDTNDATAQYW2'



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.0.0


from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import double


class CostTypes(AWSProperty):
    props = {
        'IncludeCredit': (boolean, False),
        'IncludeDiscount': (boolean, False),
        'IncludeOtherSubscription': (boolean, False),
        'IncludeRecurring': (boolean, False),
        'IncludeRefund': (boolean, False),
        'IncludeSubscription': (boolean, False),
        'IncludeSupport': (boolean, False),
        'IncludeTax': (boolean, False),
        'IncludeUpfront': (boolean, False),
        'UseAmortized': (boolean, False),
        'UseBlended': (boolean, False),
    }


class Spend(AWSProperty):
    props = {
        'Amount': (double, True),
        'Unit': (str, True),
    }


class TimePeriod(AWSProperty):
    props = {
        'End': (str, False),
        'Start': (str, False),
    }


class BudgetData(AWSProperty):
    props = {
        'BudgetLimit': (Spend, False),
        'BudgetName': (str, False),
        'BudgetType': (str, True),
        'CostFilters': (dict, False),
        'CostTypes': (CostTypes, False),
        'PlannedBudgetLimits': (dict, False),
        'TimePeriod': (TimePeriod, False),
        'TimeUnit': (str, True),
    }


class Notification(AWSProperty):
    props = {
        'ComparisonOperator': (str, True),
        'NotificationType': (str, True),
        'Threshold': (double, True),
        'ThresholdType': (str, False),
    }


class Subscriber(AWSProperty):
    props = {
        'Address': (str, True),
        'SubscriptionType': (str, True),
    }


class NotificationWithSubscribers(AWSProperty):
    props = {
        'Notification': (Notification, True),
        'Subscribers': ([Subscriber], True),
    }


class Budget(AWSObject):
    resource_type = "AWS::Budgets::Budget"

    props = {
        'Budget': (BudgetData, True),
        'NotificationsWithSubscribers':
            ([NotificationWithSubscribers], False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import integer
from .validators import integer_range
from .validators import positive_integer
from .validators import double
from .validators import json_checker


def validate_integration_type(integration_type):

    valid_integration_types = [
        "AWS",
        "AWS_PROXY",
        "HTTP",
        "HTTP_PROXY",
        "MOCK"
    ]
    if integration_type not in valid_integration_types:
        raise ValueError(
            "{} is not a valid IntegrationType".format(
                integration_type)
        )
    return integration_type


def validate_authorizer_type(authorizer_type):

    valid_authorizer_types = [
        "REQUEST"
    ]
    if authorizer_type not in valid_authorizer_types:
        raise ValueError(
            "{} is not a valid AuthorizerType".format(
                authorizer_type)
        )
    return authorizer_type


def validate_logging_level(logging_level):

    valid_logging_levels = [
        "WARN",
        "INFO",
        "DEBUG"
    ]
    if logging_level not in valid_logging_levels:
        raise ValueError(
            "{} is not a valid LoggingLevel".format(
                logging_level)
        )
    return logging_level


def validate_passthrough_behavior(passthrough_behavior):

    valid_passthrough_behaviors = [
        "WHEN_NO_MATCH",
        "WHEN_NO_TEMPLATES",
        "NEVER"
    ]
    if passthrough_behavior not in valid_passthrough_behaviors:
        raise ValueError(
            "{} is not a valid PassthroughBehavior".format(
                passthrough_behavior)
        )
    return passthrough_behavior


def validate_content_handling_strategy(content_handling_strategy):

    valid_handling_strategy_values = [
        "CONVERT_TO_TEXT",
        "CONVERT_TO_BINARY"
    ]
    if content_handling_strategy not in valid_handling_strategy_values:
        raise ValueError(
            "{} is not a valid ContentHandlingStrategy".format(
                content_handling_strategy)
        )
    return content_handling_strategy


def validate_authorizer_ttl(ttl_value):
    """ Validate authorizer ttl timeout
    :param ttl_value: The TTL timeout in seconds
    :return: The provided TTL value if valid
    """
    ttl_value = int(positive_integer(ttl_value))
    if ttl_value > 3600:
        raise ValueError("The AuthorizerResultTtlInSeconds should be <= 3600")
    return ttl_value


class BodyS3Location(AWSProperty):
    props = {
        'Bucket': (str, False),
        'Etag': (str, False),
        'Key': (str, False),
        'Version': (str, False),
    }


class Cors(AWSProperty):
    props = {
        'AllowCredentials': (boolean, False),
        'AllowHeaders': ([str], False),
        'AllowMethods': ([str], False),
        'AllowOrigins': ([str], False),
        'ExposeHeaders': ([str], False),
        'MaxAge': (integer, False),
    }


class Api(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Api"

    props = {
        'ApiKeySelectionExpression': (str, False),
        'BasePath': (str, False),
        'Body': (dict, False),
        'BodyS3Location': (BodyS3Location, False),
        'CorsConfiguration': (Cors, False),
        'CredentialsArn': (str, False),
        'Description': (str, False),
        'DisableSchemaValidation': (boolean, False),
        'FailOnWarnings': (boolean, False),
        'Name': (str, False),
        'ProtocolType': (str, False),
        'RouteKey': (str, False),
        'RouteSelectionExpression': (str, False),
        'Tags': (dict, False),
        'Target': (str, False),
        'Version': (str, False),
    }


class ApiMapping(AWSObject):
    resource_type = "AWS::ApiGatewayV2::ApiMapping"

    props = {
        'ApiId': (str, True),
        'ApiMappingKey': (str, False),
        'DomainName': (str, True),
        'Stage': (str, True),
    }


class JWTConfiguration(AWSProperty):
    props = {
        'Audience': ([str], False),
        'Issuer': (str, False),
    }


class Authorizer(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Authorizer"

    props = {
        'ApiId': (str, True),
        'AuthorizerCredentialsArn': (str, False),
        'AuthorizerResultTtlInSeconds': (validate_authorizer_ttl, False),
        'AuthorizerType': (validate_authorizer_type, True),
        'AuthorizerUri': (str, False),
        'IdentitySource': ([str], True),
        'IdentityValidationExpression': (str, False),
        'JwtConfiguration': (JWTConfiguration, False),
        'Name': (str, True),
    }


class Deployment(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Deployment"

    props = {
        'ApiId': (str, True),
        'Description': (str, False),
        'StageName': (str, False),
    }


class DomainNameConfiguration(AWSProperty):
    props = {
        'CertificateArn': (str, False),
        'CertificateName': (str, False),
        'EndpointType': (str, False),
    }


class DomainName(AWSObject):
    resource_type = "AWS::ApiGatewayV2::DomainName"

    props = {
        'DomainName': (str, True),
        'DomainNameConfigurations': ([DomainNameConfiguration], False),
        'Tags': (dict, False),
    }


class Integration(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Integration"

    props = {
        'ApiId': (str, True),
        'ConnectionType': (str, False),
        'ContentHandlingStrategy': (validate_content_handling_strategy, False),
        'CredentialsArn': (str, False),
        'Description': (str, False),
        'IntegrationMethod': (str, False),
        'IntegrationType': (validate_integration_type, True),
        'IntegrationUri': (str, False),
        'PassthroughBehavior': (validate_passthrough_behavior, False),
        'PayloadFormatVersion': (str, False),
        'RequestParameters': (dict, False),
        'RequestTemplates': (dict, False),
        'TemplateSelectionExpression': (str, False),
        'TimeoutInMillis': (integer_range(50, 29000), False),
    }


class IntegrationResponse(AWSObject):
    resource_type = "AWS::ApiGatewayV2::IntegrationResponse"

    props = {
        'ApiId': (str, True),
        'ContentHandlingStrategy': (validate_content_handling_strategy, False),
        'IntegrationId': (str, True),
        'IntegrationResponseKey': (str, True),
        'ResponseParameters': (dict, False),
        'ResponseTemplates': (dict, False),
        'TemplateSelectionExpression': (str, False),
    }


class Model(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Model"

    props = {
        'ApiId': (str, True),
        'ContentType': (str, False),
        'Description': (str, False),
        'Name': (str, True),
        'Schema': ((str, dict), True),
    }

    def validate(self):
        name = 'Schema'
        if name in self.properties:
            schema = self.properties.get(name)
            self.properties[name] = json_checker(schema)


class Route(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Route"

    props = {
        'ApiId': (str, True),
        'ApiKeyRequired': (boolean, False),
        'AuthorizationScopes': ([str], False),
        'AuthorizationType': (str, False),
        'AuthorizerId': (str, False),
        'ModelSelectionExpression': (str, False),
        'OperationName': (str, False),
        'RequestModels': (dict, False),
        'RequestParameters': (dict, False),
        'RouteKey': (str, True),
        'RouteResponseSelectionExpression': (str, False),
        'Target': (str, False),
    }


class RouteResponse(AWSObject):
    resource_type = "AWS::ApiGatewayV2::RouteResponse"

    props = {
        'ApiId': (str, True),
        'ModelSelectionExpression': (str, False),
        'ResponseModels': (dict, False),
        'ResponseParameters': (dict, False),
        'RouteId': (str, True),
        'RouteResponseKey': (str, True),
    }


class AccessLogSettings(AWSProperty):
    props = {
        'DestinationArn': (str, False),
        'Format': (str, False),
    }


class RouteSettings(AWSProperty):
    props = {
        'DataTraceEnabled': (boolean, False),
        'DetailedMetricsEnabled': (boolean, False),
        'LoggingLevel': (validate_logging_level, False),
        'ThrottlingBurstLimit': (integer, False),
        'ThrottlingRateLimit': (double, False),
    }


class Stage(AWSObject):
    resource_type = "AWS::ApiGatewayV2::Stage"

    props = {
        'AccessLogSettings': (AccessLogSettings, False),
        'ApiId': (str, True),
        'AutoDeploy': (boolean, False),
        'ClientCertificateId': (str, False),
        'DefaultRouteSettings': (RouteSettings, False),
        'DeploymentId': (str, False),
        'Description': (str, False),
        'RouteSettings': (dict, False),
        'StageName': (str, True),
        'StageVariables': (dict, False),
        'Tags': (dict, False),
    }



# Copyright (c) 2016, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags


class S3(AWSProperty):
    props = {
        'Bucket': (str, True),
        'Key': (str, True),
        'ObjectVersion': (str, False),
    }


class Code(AWSProperty):
    props = {
        'S3': (S3, True)
    }


class Trigger(AWSProperty):
    props = {
        'Branches': ([str], False),
        'CustomData': (str, False),
        'DestinationArn': (str, False),
        'Events': ([str], False),
        'Name': (str, False),
    }

    def validate(self):
        valid = [
            'all',
            'createReference',
            'deleteReference',
            'updateReference',
        ]
        events = self.properties.get('Events')
        if events and not isinstance(events, AWSHelperFn):
            if 'all' in events and len(events) != 1:
                raise ValueError('Trigger events: all must be used alone')
            else:
                for e in events:
                    if e not in valid and not isinstance(e, AWSHelperFn):
                        raise ValueError('Trigger: invalid event %s' % e)


class Repository(AWSObject):
    resource_type = "AWS::CodeCommit::Repository"

    props = {
        'Code': (Code, False),
        'RepositoryDescription': (str, False),
        'RepositoryName': (str, True),
        'Tags': (Tags, False),
        'Triggers': ([Trigger], False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean
from .validators import integer


class ServiceAccountCredentials(AWSProperty):
    props = {
        'AccountName': (str, True),
        'AccountPassword': (str, True),
    }


class DirectoryConfig(AWSObject):
    resource_type = "AWS::AppStream::DirectoryConfig"

    props = {
        'DirectoryName': (str, True),
        'OrganizationalUnitDistinguishedNames': ([str], True),
        'ServiceAccountCredentials': (ServiceAccountCredentials, True),
    }


class ComputeCapacity(AWSProperty):
    props = {
        'DesiredInstances': (integer, True),
    }


class DomainJoinInfo(AWSProperty):
    props = {
        'DirectoryName': (str, False),
        'OrganizationalUnitDistinguishedName': (str, False),
    }


class VpcConfig(AWSProperty):
    props = {
        'SecurityGroupIds': ([str], False),
        'SubnetIds': ([str], False),
    }


class Fleet(AWSObject):
    resource_type = "AWS::AppStream::Fleet"

    props = {
        'ComputeCapacity': (ComputeCapacity, True),
        'Description': (str, False),
        'DisconnectTimeoutInSeconds': (integer, False),
        'DisplayName': (str, False),
        'DomainJoinInfo': (DomainJoinInfo, False),
        'EnableDefaultInternetAccess': (boolean, False),
        'FleetType': (str, False),
        'IdleDisconnectTimeoutInSeconds': (integer, False),
        'ImageArn': (str, False),
        'ImageName': (str, False),
        'InstanceType': (str, True),
        'MaxUserDurationInSeconds': (integer, False),
        'Name': (str, False),
        'Tags': ((Tags, list), False),
        'VpcConfig': (VpcConfig, False),
    }


class AccessEndpoint(AWSProperty):
    props = {
        'EndpointType': (str, True),
        'VpceId': (str, True),
    }


class ImageBuilder(AWSObject):
    resource_type = "AWS::AppStream::ImageBuilder"

    props = {
        'AccessEndpoints': ([AccessEndpoint], False),
        'AppstreamAgentVersion': (str, False),
        'Description': (str, False),
        'DisplayName': (str, False),
        'DomainJoinInfo': (DomainJoinInfo, False),
        'EnableDefaultInternetAccess': (boolean, False),
        'ImageArn': (str, False),
        'ImageName': (str, False),
        'InstanceType': (str, True),
        'Name': (str, False),
        'Tags': ((Tags, list), False),
        'VpcConfig': (VpcConfig, False),
    }


class ApplicationSettings(AWSProperty):
    props = {
        'Enabled': (boolean, True),
        'SettingsGroup': (str, False),
    }


class StorageConnector(AWSProperty):
    props = {
        'ConnectorType': (str, True),
        'Domains': ([str], False),
        'ResourceIdentifier': (str, False),
    }


class UserSetting(AWSProperty):
    props = {
        'Action': (str, True),
        'Permission': (str, True),
    }


class Stack(AWSObject):
    resource_type = "AWS::AppStream::Stack"

    props = {
        'AccessEndpoints': ([AccessEndpoint], False),
        'ApplicationSettings': (ApplicationSettings, False),
        'AttributesToDelete': ([str], False),
        'DeleteStorageConnectors': (boolean, False),
        'Description': (str, False),
        'DisplayName': (str, False),
        'EmbedHostDomains': ([str], False),
        'FeedbackURL': (str, False),
        'Name': (str, False),
        'RedirectURL': (str, False),
        'StorageConnectors': ([StorageConnector], False),
        'Tags': ((Tags, list), False),
        'UserSettings': ([UserSetting], False),
    }


class StackFleetAssociation(AWSObject):
    resource_type = "AWS::AppStream::StackFleetAssociation"

    props = {
        'FleetName': (str, True),
        'StackName': (str, True),
    }


class StackUserAssociation(AWSObject):
    resource_type = "AWS::AppStream::StackUserAssociation"

    props = {
        'AuthenticationType': (str, True),
        'SendEmailNotification': (boolean, False),
        'StackName': (str, True),
        'UserName': (str, True),
    }


class User(AWSObject):
    resource_type = "AWS::AppStream::User"

    props = {
        'AuthenticationType': (str, True),
        'FirstName': (str, False),
        'LastName': (str, False),
        'MessageAction': (str, False),
        'UserName': (str, True),
    }



# Copyright (c) 2017, Fernando Freire <fernando.freire@nike.com>
# All rights reserved.
#
# See LICENSE file for full license.

import types

from . import AWSObject, AWSProperty
from .apigateway import AccessLogSetting, CanarySetting, MethodSetting
from .awslambda import Environment, ProvisionedConcurrencyConfiguration
from .awslambda import VPCConfig, validate_memory_size
from .dynamodb import ProvisionedThroughput, SSESpecification
from .s3 import Filter
from .validators import exactly_one, positive_integer, mutually_exclusive
try:
    from awacs.aws import PolicyDocument
    policytypes = (dict, list, str, PolicyDocument)
except ImportError:
    policytypes = (dict, list, str)

assert types  # silence pyflakes


def primary_key_type_validator(x):
    valid_types = ["String", "Number", "Binary"]
    if x not in valid_types:
        raise ValueError("KeyType must be one of: %s" % ", ".join(valid_types))
    return x


class DeadLetterQueue(AWSProperty):
    props = {
        'Type': (str, False),
        'TargetArn': (str, False)
    }

    def validate(self):
        valid_types = ['SQS', 'SNS']
        if ('Type' in self.properties and
                self.properties['Type'] not in valid_types):
            raise ValueError('Type must be either SQS or SNS')


class S3Location(AWSProperty):
    props = {
        "Bucket": (str, True),
        "Key": (str, True),
        "Version": (str, False),
    }


class Hooks(AWSProperty):
    props = {
        "PreTraffic": (str, False),
        "PostTraffic": (str, False),
    }


class DeploymentPreference(AWSProperty):
    props = {
        "Type": (str, True),
        "Alarms": (list, False),
        "Hooks": (Hooks, False),
        "Enabled": (bool, False),
        "Role": (str, False),
    }


class Function(AWSObject):
    resource_type = "AWS::Serverless::Function"

    props = {
        'Handler': (str, True),
        'Runtime': (str, True),
        'CodeUri': ((S3Location, str), False),
        'InlineCode': (str, False),
        'FunctionName': (str, False),
        'Description': (str, False),
        'MemorySize': (validate_memory_size, False),
        'Timeout': (positive_integer, False),
        'Role': (str, False),
        'Policies': (policytypes, False),
        'Environment': (Environment, False),
        'VpcConfig': (VPCConfig, False),
        'Events': (dict, False),
        'Tags': (dict, False),
        'Tracing': (str, False),
        'KmsKeyArn': (str, False),
        'DeadLetterQueue': (DeadLetterQueue, False),
        'DeploymentPreference': (DeploymentPreference, False),
        'Layers': ([str], False),
        'AutoPublishAlias': (str, False),
        'ReservedConcurrentExecutions': (positive_integer, False),
        'ProvisionedConcurrencyConfig':
            (ProvisionedConcurrencyConfiguration, False),
    }

    def validate(self):
        conds = [
            'CodeUri',
            'InlineCode',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class FunctionForPackaging(Function):
    """Render Function without requiring 'CodeUri'.

    This exception to the Function spec is for use with the
    `cloudformation/sam package` commands which add CodeUri automatically.
    """

    resource_type = Function.resource_type
    props = Function.props.copy()
    props['CodeUri'] = (props['CodeUri'][0], False)

    def validate(self):
        pass


class CognitoAuthIdentity(AWSProperty):
    props = {
        'Header': (str, False),
        'ValidationExpression': (str, False),
    }


class LambdaTokenAuthIdentity(AWSProperty):
    props = {
        'Header': (str, False),
        'ValidationExpression': (str, False),
        'ReauthorizeEvery': (str, False),
    }


class LambdaRequestAuthIdentity(AWSProperty):
    props = {
        'Headers': ([str], False),
        'QueryStrings': ([str], False),
        'StageVariables': ([str], False),
        'Context': ([str], False),
        'ReauthorizeEvery': (str, False),
    }


class CognitoAuth(AWSProperty):
    props = {
        'UserPoolArn': (str, False),
        'Identity': (CognitoAuthIdentity, False),
    }


class LambdaTokenAuth(AWSProperty):
    props = {
        'FunctionPayloadType': (str, False),
        'FunctionArn': (str, False),
        'FunctionInvokeRole': (str, False),
        'Identity': (LambdaTokenAuthIdentity, False),
    }


class LambdaRequestAuth(AWSProperty):
    props = {
        'FunctionPayloadType': (str, False),
        'FunctionArn': (str, False),
        'FunctionInvokeRole': (str, False),
        'Identity': (LambdaRequestAuthIdentity, False),
    }


class Authorizers(AWSProperty):
    props = {
        'DefaultAuthorizer': (str, False),
        'CognitoAuth': (CognitoAuth, False),
        'LambdaTokenAuth': (LambdaTokenAuth, False),
        'LambdaRequestAuth': (LambdaRequestAuth, False),
    }


class ResourcePolicyStatement(AWSProperty):
    props = {
        'AwsAccountBlacklist': (list, False),
        'AwsAccountWhitelist': (list, False),
        'CustomStatements': (list, False),
        'IpRangeBlacklist': (list, False),
        'IpRangeWhitelist': (list, False),
        'SourceVpcBlacklist': (list, False),
        'SourceVpcWhitelist': (list, False),
    }


class Auth(AWSProperty):
    props = {
        'AddDefaultAuthorizerToCorsPreflight': (bool, False),
        'ApiKeyRequired': (bool, False),
        'Authorizers': (Authorizers, False),
        'DefaultAuthorizer': (str, False),
        'InvokeRole': (str, False),
        'ResourcePolicy': (ResourcePolicyStatement, False),
    }


class Cors(AWSProperty):
    props = {
        'AllowCredentials': (str, False),
        'AllowHeaders': (str, False),
        'AllowMethods': (str, False),
        'AllowOrigin': (str, True),
        'MaxAge': (str, False),
    }


class Api(AWSObject):
    resource_type = "AWS::Serverless::Api"

    props = {
        'AccessLogSetting': (AccessLogSetting, False),
        'Auth': (Auth, False),
        'BinaryMediaTypes': ([str], False),
        'CacheClusterEnabled': (bool, False),
        'CacheClusterSize': (str, False),
        'CanarySetting': (CanarySetting, False),
        'Cors': ((str, Cors), False),
        'DefinitionBody': (dict, False),
        'DefinitionUri': (str, False),
        'EndpointConfiguration': (str, False),
        'MethodSettings': ([MethodSetting], False),
        'Name': (str, False),
        'StageName': (str, True),
        "TracingEnabled": (bool, False),
        'Variables': (dict, False),
    }

    def validate(self):
        conds = [
            'DefinitionBody',
            'DefinitionUri',
        ]
        mutually_exclusive(self.__class__.__name__, self.properties, conds)


class PrimaryKey(AWSProperty):
    props = {
        'Name': (str, False),
        'Type': (primary_key_type_validator, False)
    }


class SimpleTable(AWSObject):
    resource_type = "AWS::Serverless::SimpleTable"

    props = {
        'PrimaryKey': (PrimaryKey, False),
        'ProvisionedThroughput': (ProvisionedThroughput, False),
        'SSESpecification': (SSESpecification, False),
        'Tags': (dict, False),
        'TableName': (str, False),
    }


class LayerVersion(AWSObject):
    resource_type = "AWS::Serverless::LayerVersion"

    props = {
        'CompatibleRuntimes': ([str], False),
        'ContentUri': ((S3Location, str), True),
        'Description': (str, False),
        'LayerName': (str, False),
        'LicenseInfo': (str, False),
        'RetentionPolicy': (str, False),
    }


class S3Event(AWSObject):
    resource_type = 'S3'

    props = {
        'Bucket': (str, True),
        'Events': (list, True),
        'Filter': (Filter, False)
    }


class SNSEvent(AWSObject):
    resource_type = 'SNS'

    props = {
        'Topic': (str, True)
    }


def starting_position_validator(x):
    valid_types = ['TRIM_HORIZON', 'LATEST']
    if x not in valid_types:
        raise ValueError(
            "StartingPosition must be one of: %s"
            % ", ".join(valid_types)
        )
    return x


class KinesisEvent(AWSObject):
    resource_type = 'Kinesis'

    props = {
        'Stream': (str, True),
        'StartingPosition': (starting_position_validator, True),
        'BatchSize': (positive_integer, False)
    }


class DynamoDBEvent(AWSObject):
    resource_type = 'DynamoDB'

    props = {
        'Stream': (str, True),
        'StartingPosition': (starting_position_validator, True),
        'BatchSize': (positive_integer, False)
    }


class ApiEvent(AWSObject):
    resource_type = 'Api'

    props = {
        'Path': (str, True),
        'Method': (str, True),
        'RestApiId': (str, False)
    }


class ScheduleEvent(AWSObject):
    resource_type = 'Schedule'

    props = {
        'Schedule': (str, True),
        'Input': (str, False)
    }


class CloudWatchEvent(AWSObject):
    resource_type = 'CloudWatchEvent'

    props = {
        'Pattern': (dict, True),
        'Input': (str, False),
        'InputPath': (str, False)
    }


class IoTRuleEvent(AWSObject):
    resource_type = 'IoTRule'

    props = {
        'Sql': (str, True),
        'AwsIotSqlVersion': (str, False)
    }


class AlexaSkillEvent(AWSObject):
    resource_type = 'AlexaSkill'
    props = {}


class SQSEvent(AWSObject):
    resource_type = 'SQS'

    props = {
        'Queue': (str, True),
        'BatchSize': (positive_integer, True)
    }

    def validate(self):
        if (not 1 <= self.properties['BatchSize'] <= 10):
            raise ValueError('BatchSize must be between 1 and 10')



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import (boolean, double, integer, scalable_dimension_type,
                         service_namespace_type, statistic_type)


VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR = (
    'SetForecastCapacityToMaxCapacity',
    'SetMaxCapacityToForecastCapacity',
    'SetMaxCapacityAboveForecastCapacity',
)
VALID_PREDICTIVESCALINGMODE = ('ForecastAndScale', 'ForecastOnly')
VALID_SCALINGPOLICYUPDATEBEHAVIOR = ('KeepExternalPolicies',
                                     'ReplaceExternalPolicies')


def validate_predictivescalingmaxcapacitybehavior(
        predictivescalingmaxcapacitybehavior):
    """Validate PredictiveScalingMaxCapacityBehavior for ScalingInstruction"""  # noqa

    if predictivescalingmaxcapacitybehavior not in VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR:  # noqa
        raise ValueError("ScalingInstruction PredictiveScalingMaxCapacityBehavior must be one of: %s" %  # noqa
                         ", ".join(VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR))
    return predictivescalingmaxcapacitybehavior


def validate_predictivescalingmode(predictivescalingmode):
    """Validate PredictiveScalingMode for ScalingInstruction"""

    if predictivescalingmode not in VALID_PREDICTIVESCALINGMODE:
        raise ValueError("ScalingInstruction PredictiveScalingMode must be one of: %s" %  # noqa
                         ", ".join(VALID_PREDICTIVESCALINGMODE))
    return predictivescalingmode


def validate_scalingpolicyupdatebehavior(scalingpolicyupdatebehavior):
    """Validate ScalingPolicyUpdateBehavior for ScalingInstruction"""

    if scalingpolicyupdatebehavior not in VALID_SCALINGPOLICYUPDATEBEHAVIOR:
        raise ValueError("ScalingInstruction ScalingPolicyUpdateBehavior must be one of: %s" %  # noqa
                         ", ".join(VALID_SCALINGPOLICYUPDATEBEHAVIOR))
    return scalingpolicyupdatebehavior


class TagFilter(AWSProperty):
    props = {
        'Values': ([str], False),
        'Key': (str, True)
    }


class ApplicationSource(AWSProperty):
    props = {
        'CloudFormationStackARN': (str, False),
        'TagFilters': ([TagFilter], False)
    }


class PredefinedScalingMetricSpecification(AWSProperty):
    props = {
        'ResourceLabel': (str, False),
        'PredefinedScalingMetricType': (str, True)
    }


class MetricDimension(AWSProperty):
    props = {
        'Value': (str, True),
        'Name': (str, True)
    }


class CustomizedScalingMetricSpecification(AWSProperty):
    props = {
        'MetricName': (str, True),
        'Statistic': (statistic_type, True),
        'Dimensions': ([MetricDimension], False),
        'Unit': (str, False),
        'Namespace': (str, True)
    }


class TargetTrackingConfiguration(AWSProperty):
    props = {
        'ScaleOutCooldown': (integer, False),
        'TargetValue': (double, True),
        'PredefinedScalingMetricSpecification': (
            PredefinedScalingMetricSpecification,
            False
        ),
        'DisableScaleIn': (boolean, False),
        'ScaleInCooldown': (integer, False),
        'EstimatedInstanceWarmup': (integer, False),
        'CustomizedScalingMetricSpecification': (
            CustomizedScalingMetricSpecification,
            False
        )
    }


class CustomizedLoadMetricSpecification(AWSObject):
    props = {
        'Dimensions': ([MetricDimension], False),
        'MetricName': (str, True),
        'Namespace': (str, True),
        'Statistic': (str, True),
        'Unit': (str, False),
    }


class PredefinedLoadMetricSpecification(AWSProperty):
    props = {
        'PredefinedLoadMetricType': (str, True),
        'ResourceLabel': (str, False),
    }


class ScalingInstruction(AWSProperty):
    props = {
        'CustomizedLoadMetricSpecification': (CustomizedLoadMetricSpecification, False),  # NOQA
        'DisableDynamicScaling': (boolean, False),
        'MaxCapacity': (integer, True),
        'MinCapacity': (integer, True),
        'PredefinedLoadMetricSpecification': (PredefinedLoadMetricSpecification, False),  # NOQA
        'PredictiveScalingMaxCapacityBehavior': (validate_predictivescalingmaxcapacitybehavior, False),  # NOQA
        'PredictiveScalingMaxCapacityBuffer': (integer, False),
        'PredictiveScalingMode': (validate_predictivescalingmode, False),
        'ResourceId': (str, True),
        'ScalableDimension': (scalable_dimension_type, True),
        'ScalingPolicyUpdateBehavior': (validate_scalingpolicyupdatebehavior, False),  # NOQA
        'ScheduledActionBufferTime': (integer, False),
        'ServiceNamespace': (service_namespace_type, True),
        'TargetTrackingConfigurations': (
            [TargetTrackingConfiguration],
            True
        ),
    }


class ScalingPlan(AWSObject):
    resource_type = "AWS::AutoScalingPlans::ScalingPlan"

    props = {
        'ApplicationSource': (ApplicationSource, True),
        'ScalingInstructions': ([ScalingInstruction], True)
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class ActionTypeId(AWSProperty):
    props = {
        'Category': (str, True),
        'Owner': (str, True),
        'Provider': (str, True),
        'Version': (str, True)
    }


class ArtifactDetails(AWSProperty):
    props = {
        'MaximumCount': (integer, True),
        'MinimumCount': (integer, True)
    }


class Blockers(AWSProperty):
    props = {
        'Name': (str, True),
        'Type': (str, True)
    }


class ConfigurationProperties(AWSProperty):
    props = {
        'Description': (str, False),
        'Key': (boolean, True),
        'Name': (str, True),
        'Queryable': (boolean, False),
        'Required': (boolean, True),
        'Secret': (boolean, True),
        'Type': (str, False)
    }


class EncryptionKey(AWSProperty):
    props = {
        'Id': (str, True),
        'Type': (str, True)
    }


class DisableInboundStageTransitions(AWSProperty):
    props = {
        'Reason': (str, True),
        'StageName': (str, True)
    }


class InputArtifacts(AWSProperty):
    props = {
        'Name': (str, True)
    }


class OutputArtifacts(AWSProperty):
    props = {
        'Name': (str, True)
    }


class Settings(AWSProperty):
    props = {
        'EntityUrlTemplate': (str, False),
        'ExecutionUrlTemplate': (str, False),
        'RevisionUrlTemplate': (str, False),
        'ThirdPartyConfigurationUrl': (str, False)
    }


class ArtifactStore(AWSProperty):
    props = {
        'EncryptionKey': (EncryptionKey, False),
        'Location': (str, True),
        'Type': (str, True)
    }


class ArtifactStoreMap(AWSProperty):
    props = {
        'ArtifactStore': (ArtifactStore, True),
        'Region': (str, True)
    }


class Actions(AWSProperty):
    props = {
        'ActionTypeId': (ActionTypeId, True),
        'Configuration': (dict, False),
        'InputArtifacts': ([InputArtifacts], False),
        'Name': (str, True),
        'Namespace': (str, False),
        'OutputArtifacts': ([OutputArtifacts], False),
        'Region': (str, False),
        'RoleArn': (str, False),
        'RunOrder': (integer, False)
    }


class Stages(AWSProperty):
    props = {
        'Actions': ([Actions], True),
        'Blockers': ([Blockers], False),
        'Name': (str, True)
    }


class CustomActionType(AWSObject):
    resource_type = "AWS::CodePipeline::CustomActionType"

    props = {
        'Category': (str, True),
        'ConfigurationProperties': ([ConfigurationProperties], False),
        'InputArtifactDetails': (ArtifactDetails, True),
        'OutputArtifactDetails': (ArtifactDetails, True),
        'Provider': (str, True),
        'Settings': (Settings, False),
        'Tags': (Tags, False),
        'Version': (str, False),
    }


class Pipeline(AWSObject):
    resource_type = "AWS::CodePipeline::Pipeline"

    props = {
        'ArtifactStore': (ArtifactStore, False),
        'ArtifactStores': ([ArtifactStoreMap], False),
        'DisableInboundStageTransitions':
            ([DisableInboundStageTransitions], False),
        'Name': (str, False),
        'RestartExecutionOnUpdate': (boolean, False),
        'RoleArn': (str, True),
        'Stages': ([Stages], True),
        'Tags': (Tags, False),
    }


class WebhookAuthConfiguration(AWSProperty):
    props = {
        'AllowedIPRange': (str, False),
        'SecretToken': (str, False),
    }


class WebhookFilterRule(AWSProperty):
    props = {
        'JsonPath': (str, True),
        'MatchEquals': (str, False),
    }


class Webhook(AWSObject):
    resource_type = "AWS::CodePipeline::Webhook"

    props = {
        'Authentication': (str, True),
        'AuthenticationConfiguration': (WebhookAuthConfiguration, True),
        'Filters': ([WebhookFilterRule], True),
        'Name': (str, False),
        'RegisterWithThirdParty': (boolean, False),
        'TargetAction': (str, True),
        'TargetPipeline': (str, True),
        'TargetPipelineVersion': (integer, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, network_port, positive_integer


CDC = "cdc"
FULL_LOAD = "full-load"
FULL_LOAD_AND_CDC = "full-load-and-cdc"


class Certificate(AWSObject):
    resource_type = "AWS::DMS::Certificate"

    props = {
        'CertificateIdentifier': (str, False),
        'CertificatePem': (str, False),
        'CertificateWallet': (str, False),
    }


class DynamoDbSettings(AWSProperty):
    props = {
        'ServiceAccessRoleArn': (str, False),
    }


class ElasticsearchSettings(AWSProperty):
    props = {
        'EndpointUri': (str, False),
        'ErrorRetryDuration': (integer, False),
        'FullLoadErrorPercentage': (integer, False),
        'ServiceAccessRoleArn': (str, False),
    }


class KinesisSettings(AWSProperty):
    props = {
        'MessageFormat': (str, False),
        'ServiceAccessRoleArn': (str, False),
        'StreamArn': (str, False),
    }


class MongoDbSettings(AWSProperty):
    props = {
        'AuthMechanism': (str, False),
        'AuthSource': (str, False),
        'AuthType': (str, False),
        'DatabaseName': (str, False),
        'DocsToInvestigate': (str, False),
        'ExtractDocId': (str, False),
        'NestingLevel': (str, False),
        'Password': (str, False),
        'Port': (network_port, False),
        'ServerName': (str, False),
        'Username': (str, False),
    }


class S3Settings(AWSProperty):
    props = {
        'BucketFolder': (str, False),
        'BucketName': (str, False),
        'CompressionType': (str, False),
        'CsvDelimiter': (str, False),
        'CsvRowDelimiter': (str, False),
        'ExternalTableDefinition': (str, False),
        'ServiceAccessRoleArn': (str, False),
    }


class Endpoint(AWSObject):
    resource_type = "AWS::DMS::Endpoint"

    props = {
        'CertificateArn': (str, False),
        'DatabaseName': (str, False),
        'DynamoDbSettings': (DynamoDbSettings, False),
        'ElasticsearchSettings': (ElasticsearchSettings, False),
        'EndpointIdentifier': (str, False),
        'EndpointType': (str, True),
        'EngineName': (str, True),
        'ExtraConnectionAttributes': (str, False),
        'KinesisSettings': (KinesisSettings, False),
        'KmsKeyId': (str, False),
        'MongoDbSettings': (MongoDbSettings, False),
        'Password': (str, False),
        'Port': (network_port, False),
        'S3Settings': (S3Settings, False),
        'ServerName': (str, False),
        'SslMode': (str, False),
        'Tags': (Tags, False),
        'Username': (str, False),
    }


class EventSubscription(AWSObject):
    resource_type = "AWS::DMS::EventSubscription"

    props = {
        'Enabled': (boolean, False),
        'EventCategories': ([str], False),
        'SnsTopicArn': (str, True),
        'SourceIds': ([str], False),
        'SourceType': (str, False),
        'SubscriptionName': (str, False),
        'Tags': (Tags, False),
    }


class ReplicationInstance(AWSObject):
    resource_type = "AWS::DMS::ReplicationInstance"

    props = {
        'AllocatedStorage': (integer, False),
        'AllowMajorVersionUpgrade': (boolean, False),
        'AutoMinorVersionUpgrade': (boolean, False),
        'AvailabilityZone': (str, False),
        'EngineVersion': (str, False),
        'KmsKeyId': (str, False),
        'MultiAZ': (boolean, False),
        'PreferredMaintenanceWindow': (str, False),
        'PubliclyAccessible': (boolean, False),
        'ReplicationInstanceClass': (str, True),
        'ReplicationInstanceIdentifier': (str, False),
        'ReplicationSubnetGroupIdentifier': (str, False),
        'Tags': (Tags, False),
        'VpcSecurityGroupIds': ([str], False),
    }


class ReplicationSubnetGroup(AWSObject):
    resource_type = "AWS::DMS::ReplicationSubnetGroup"

    props = {
        'ReplicationSubnetGroupDescription': (str, True),
        'ReplicationSubnetGroupIdentifier': (str, False),
        'SubnetIds': ([str], True),
        'Tags': (Tags, False),
    }


class ReplicationTask(AWSObject):
    resource_type = "AWS::DMS::ReplicationTask"

    props = {
        'CdcStartPosition': (str, False),
        'CdcStartTime': (positive_integer, False),
        'CdcStopPosition': (str, False),
        'MigrationType': (str, True),
        'ReplicationInstanceArn': (str, True),
        'ReplicationTaskIdentifier': (str, False),
        'ReplicationTaskSettings': (str, False),
        'SourceEndpointArn': (str, True),
        'TableMappings': (str, True),
        'Tags': (Tags, False),
        'TargetEndpointArn': (str, True),
    }



from . import AWSObject, AWSProperty
from .validators import boolean, json_checker


class Device(AWSObject):
    resource_type = "AWS::IoT1Click::Device"

    props = {
        'DeviceId': (str, True),
        'Enabled': (boolean, True),
    }


class Placement(AWSObject):
    resource_type = "AWS::IoT1Click::Placement"

    props = {
        'AssociatedDevices': (json_checker, False),
        'Attributes': (json_checker, False),
        'PlacementName': (str, False),
        'ProjectName': (str, True),
    }


class PlacementTemplate(AWSProperty):
    props = {
        'DefaultAttributes': (json_checker, False),
        'DeviceTemplates': (json_checker, False),
    }


class Project(AWSObject):
    resource_type = "AWS::IoT1Click::Project"

    props = {
        'Description': (str, False),
        'PlacementTemplate': (PlacementTemplate, True),
        'ProjectName': (str, False),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject


class NamedQuery(AWSObject):
    resource_type = "AWS::Athena::NamedQuery"

    props = {
        'Database': (str, True),
        'Description': (str, False),
        'Name': (str, False),
        'QueryString': (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 6.1.0


from . import AWSObject
from troposphere import Tags
from .validators import boolean


class Ledger(AWSObject):
    resource_type = "AWS::QLDB::Ledger"

    props = {
        'DeletionProtection': (boolean, False),
        'Name': (str, False),
        'PermissionsMode': (str, True),
        'Tags': (Tags, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject
from . import AWSProperty
from .validators import integer


class RoutingStrategy(AWSProperty):
    props = {
        'FleetId': (str, False),
        'Message': (str, False),
        'Type': (str, True),
    }


class Alias(AWSObject):
    resource_type = "AWS::GameLift::Alias"

    props = {
        'Description': (str, False),
        'Name': (str, True),
        'RoutingStrategy': (RoutingStrategy, True),
    }


class S3Location(AWSProperty):
    props = {
        'Bucket': (str, True),
        'Key': (str, True),
        'ObjectVersion': (str, False),
        'RoleArn': (str, True),
    }


class Build(AWSObject):
    resource_type = "AWS::GameLift::Build"

    props = {
        'Name': (str, False),
        'OperatingSystem': (str, False),
        'StorageLocation': (S3Location, False),
        'Version': (str, False),
    }


class CertificateConfiguration(AWSProperty):
    props = {
        'CertificateType': (str, True),
    }


class IpPermission(AWSProperty):
    props = {
        'FromPort': (integer, True),
        'IpRange': (str, True),
        'Protocol': (str, True),
        'ToPort': (integer, True),
    }


class ResourceCreationLimitPolicy(AWSProperty):
    props = {
        'NewGameSessionsPerCreator': (integer, False),
        'PolicyPeriodInMinutes': (integer, False),
    }


class ServerProcess(AWSProperty):
    props = {
        'ConcurrentExecutions': (integer, True),
        'LaunchPath': (str, True),
        'Parameters': (str, False),
    }


class RuntimeConfiguration(AWSProperty):
    props = {
        'GameSessionActivationTimeoutSeconds': (integer, False),
        'MaxConcurrentGameSessionActivations': (integer, False),
        'ServerProcesses': ([ServerProcess], False),
    }


class Fleet(AWSObject):
    resource_type = "AWS::GameLift::Fleet"

    props = {
        'BuildId': (str, False),
        'CertificateConfiguration': (CertificateConfiguration, False),
        'Description': (str, False),
        'DesiredEC2Instances': (integer, False),
        'EC2InboundPermissions': ([IpPermission], False),
        'EC2InstanceType': (str, True),
        'FleetType': (str, False),
        'InstanceRoleARN': (str, False),
        'LogPaths': ([str], False),
        'MaxSize': (integer, False),
        'MetricGroups': ([str], False),
        'MinSize': (integer, False),
        'Name': (str, True),
        'NewGameSessionProtectionPolicy': (str, False),
        'PeerVpcAwsAccountId': (str, False),
        'PeerVpcId': (str, False),
        'ResourceCreationLimitPolicy': (ResourceCreationLimitPolicy, False),
        'RuntimeConfiguration': (RuntimeConfiguration, False),
        'ScriptId': (str, False),
        'ServerLaunchParameters': (str, False),
        'ServerLaunchPath': (str, False),
    }



# Copyright (c) 2016-2017, troposphere project
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer, positive_integer


def processor_type_validator(x):
    valid_types = ["Lambda"]
    if x not in valid_types:
        raise ValueError("Type must be one of: %s" %
                         ", ".join(valid_types))
    return x


def delivery_stream_type_validator(x):
    valid_types = ["DirectPut", "KinesisStreamAsSource"]
    if x not in valid_types:
        raise ValueError("DeliveryStreamType must be one of: %s" %
                         ", ".join(valid_types))
    return x


def index_rotation_period_validator(x):
    valid_types = ["NoRotation", "OneHour", "OneDay", "OneWeek", "OneMonth"]
    if x not in valid_types:
        raise ValueError("IndexRotationPeriod must be one of: %s" %
                         ", ".join(valid_types))
    return x


def s3_backup_mode_elastic_search_validator(x):
    valid_types = ["FailedDocumentsOnly", "AllDocuments"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" %
                         ", ".join(valid_types))
    return x


def s3_backup_mode_extended_s3_validator(x):
    valid_types = ["Disabled", "Enabled"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" %
                         ", ".join(valid_types))
    return x


class BufferingHints(AWSProperty):
    props = {
        'IntervalInSeconds': (positive_integer, True),
        'SizeInMBs': (positive_integer, True)
    }


class CloudWatchLoggingOptions(AWSProperty):
    props = {
        'Enabled': (boolean, False),
        'LogGroupName': (str, False),  # Conditional
        'LogStreamName': (str, False),  # Conditional
    }


class RetryOptions(AWSProperty):
    props = {
        'DurationInSeconds': (positive_integer, True),
    }


class KMSEncryptionConfig(AWSProperty):
    props = {
        'AWSKMSKeyARN': (str, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        'KMSEncryptionConfig': (KMSEncryptionConfig, False),
        'NoEncryptionConfig': (str, False),
    }


class S3Configuration(AWSProperty):
    props = {
        'BucketARN': (str, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (str, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'Prefix': (str, False),
        'RoleARN': (str, True)
    }


class CopyCommand(AWSProperty):
    props = {
        'CopyOptions': (str, False),
        'DataTableColumns': (str, False),
        'DataTableName': (str, True),
    }


class ProcessorParameter(AWSProperty):
    props = {
        'ParameterName': (str, True),
        'ParameterValue': (str, True),
    }


class Processor(AWSProperty):
    props = {
        'Parameters': ([ProcessorParameter], True),
        'Type': (processor_type_validator, True),
    }


class ProcessingConfiguration(AWSProperty):
    props = {
        'Enabled': (boolean, True),
        'Processors': ([Processor], True),
    }


class ElasticsearchDestinationConfiguration(AWSProperty):
    props = {
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'DomainARN': (str, True),
        'IndexName': (str, True),
        'IndexRotationPeriod': (index_rotation_period_validator, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RetryOptions': (RetryOptions, False),
        'RoleARN': (str, True),
        'S3BackupMode': (s3_backup_mode_elastic_search_validator, True),
        'S3Configuration': (S3Configuration, False),
        'TypeName': (str, True),
    }


class RedshiftDestinationConfiguration(AWSProperty):
    props = {
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'ClusterJDBCURL': (str, True),
        'CopyCommand': (CopyCommand, True),
        'Password': (str, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RoleARN': (str, True),
        'S3Configuration': (S3Configuration, True),
        'Username': (str, True),
    }


class S3DestinationConfiguration(AWSProperty):
    props = {
        'BucketARN': (str, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (str, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'ErrorOutputPrefix': (str, False),
        'Prefix': (str, False),
        'RoleARN': (str, True),
    }


class HiveJsonSerDe(AWSProperty):
    props = {
        'TimestampFormats': ([str], False),
    }


class OpenXJsonSerDe(AWSProperty):
    props = {
        'CaseInsensitive': (boolean, False),
        'ColumnToJsonKeyMappings': (dict, False),
        'ConvertDotsInJsonKeysToUnderscores': (boolean, False),
    }


class Deserializer(AWSProperty):
    props = {
        'HiveJsonSerDe': (HiveJsonSerDe, False),
        'OpenXJsonSerDe': (OpenXJsonSerDe, False),
    }


class InputFormatConfiguration(AWSProperty):
    props = {
        'Deserializer': (Deserializer, True),
    }


class OrcSerDe(AWSProperty):
    props = {
        'BlockSizeBytes': (integer, False),
        'BloomFilterColumns': ([str], False),
        'BloomFilterFalsePositiveProbability': (float, False),
        'Compression': (str, False),
        'DictionaryKeyThreshold': (float, False),
        'EnablePadding': (boolean, False),
        'FormatVersion': (str, False),
        'PaddingTolerance': (float, False),
        'RowIndexStride': (integer, False),
        'StripeSizeBytes': (integer, False),
    }


class ParquetSerDe(AWSProperty):
    props = {
        'BlockSizeBytes': (integer, False),
        'Compression': (str, False),
        'EnableDictionaryCompression': (boolean, False),
        'MaxPaddingBytes': (integer, False),
        'PageSizeBytes': (integer, False),
        'WriterVersion': (str, False),
    }


class Serializer(AWSProperty):
    props = {
        'OrcSerDe': (OrcSerDe, False),
        'ParquetSerDe': (ParquetSerDe, False),
    }


class OutputFormatConfiguration(AWSProperty):
    props = {
        'Serializer': (Serializer, True),
    }


class SchemaConfiguration(AWSProperty):
    props = {
        'CatalogId': (str, True),
        'DatabaseName': (str, True),
        'Region': (str, True),
        'RoleARN': (str, True),
        'TableName': (str, True),
        'VersionId': (str, True),
    }


class DataFormatConversionConfiguration(AWSProperty):
    props = {
        'Enabled': (boolean, True),
        'InputFormatConfiguration': (InputFormatConfiguration, True),
        'OutputFormatConfiguration': (OutputFormatConfiguration, True),
        'SchemaConfiguration': (SchemaConfiguration, True),
    }


class ExtendedS3DestinationConfiguration(AWSProperty):
    props = {
        'BucketARN': (str, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (str, True),
        'DataFormatConversionConfiguration':
            (DataFormatConversionConfiguration, False),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'ErrorOutputPrefix': (str, False),
        'Prefix': (str, False),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RoleARN': (str, True),
        'S3BackupConfiguration': (S3DestinationConfiguration, False),
        'S3BackupMode': (s3_backup_mode_extended_s3_validator, False),
    }


class KinesisStreamSourceConfiguration(AWSProperty):
    props = {
        'KinesisStreamARN': (str, True),
        'RoleARN': (str, True)
    }


class SplunkRetryOptions(AWSProperty):
    props = {
        'DurationInSeconds': (positive_integer, True),
    }


class SplunkDestinationConfiguration(AWSProperty):
    props = {
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'HECAcknowledgmentTimeoutInSeconds': (positive_integer, False),
        'HECEndpoint': (str, True),
        'HECEndpointType': (str, True),
        'HECToken': (str, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RetryOptions': (SplunkRetryOptions, False),
        'S3BackupMode': (str, False),
        'S3Configuration': (S3DestinationConfiguration, True),
    }


class DeliveryStream(AWSObject):
    resource_type = "AWS::KinesisFirehose::DeliveryStream"

    props = {
        'DeliveryStreamName': (str, False),
        'DeliveryStreamType': (delivery_stream_type_validator, False),
        'ElasticsearchDestinationConfiguration': (ElasticsearchDestinationConfiguration, False),  # noqa
        'ExtendedS3DestinationConfiguration': (ExtendedS3DestinationConfiguration, False),  # noqa
        'KinesisStreamSourceConfiguration': (KinesisStreamSourceConfiguration, False),  # noqa
        'RedshiftDestinationConfiguration': (RedshiftDestinationConfiguration, False),  # noqa
        'S3DestinationConfiguration': (S3DestinationConfiguration, False),
        'SplunkDestinationConfiguration':
            (SplunkDestinationConfiguration, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject
from . import AWSProperty
from troposphere import Tags


class Fleet(AWSObject):
    resource_type = "AWS::RoboMaker::Fleet"

    props = {
        'Name': (str, False),
        'Tags': (Tags, False),
    }


class Robot(AWSObject):
    resource_type = "AWS::RoboMaker::Robot"

    props = {
        'Architecture': (str, True),
        'Fleet': (str, False),
        'GreengrassGroupId': (str, True),
        'Name': (str, False),
        'Tags': (Tags, False),
    }


class RobotSoftwareSuite(AWSProperty):
    props = {
        'Name': (str, True),
        'Version': (str, True),
    }


class SourceConfig(AWSProperty):
    props = {
        'Architecture': (str, True),
        'S3Bucket': (str, True),
        'S3Key': (str, True),
    }


class RobotApplication(AWSObject):
    resource_type = "AWS::RoboMaker::RobotApplication"

    props = {
        'CurrentRevisionId': (str, False),
        'Name': (str, False),
        'RobotSoftwareSuite': (RobotSoftwareSuite, True),
        'Sources': ([SourceConfig], True),
        'Tags': (Tags, False),
    }


class RobotApplicationVersion(AWSObject):
    resource_type = "AWS::RoboMaker::RobotApplicationVersion"

    props = {
        'Application': (str, True),
        'CurrentRevisionId': (str, False),
    }


class RenderingEngine(AWSProperty):
    props = {
        'Name': (str, True),
        'Version': (str, True),
    }


class SimulationSoftwareSuite(AWSProperty):
    props = {
        'Name': (str, True),
        'Version': (str, True),
    }


class SimulationApplication(AWSObject):
    resource_type = "AWS::RoboMaker::SimulationApplication"

    props = {
        'CurrentRevisionId': (str, False),
        'Name': (str, False),
        'RenderingEngine': (RenderingEngine, True),
        'RobotSoftwareSuite': (RobotSoftwareSuite, True),
        'SimulationSoftwareSuite': (SimulationSoftwareSuite, True),
        'Sources': ([SourceConfig], True),
        'Tags': (Tags, False),
    }


class SimulationApplicationVersion(AWSObject):
    resource_type = "AWS::RoboMaker::SimulationApplicationVersion"

    props = {
        'Application': (str, True),
        'CurrentRevisionId': (str, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# Copyright (c) 2014, Andy Botting <andy.botting@theguardian.com>
# All rights reserved.
#
# See LICENSE file for full license.


from troposphere import AWSObject, AWSProperty
from troposphere.validators import boolean, integer, network_port


class BlockDeviceMapping(AWSProperty):
    props = {
        'delete_on_termination': (boolean, False),
        'device_name': (str, True),
        'snapshot_id': (str, False),
        'volume_id': (str, False),
        'volume_size': (integer, False),
    }


class BlockDeviceMappingV2(AWSProperty):
    props = {
        'boot_index': (integer, False),
        'delete_on_termination': (boolean, False),
        'device_name': (str, False),
        'device_type': (str, False),
        'disk_bus': (str, False),
        'ephemeral_format': (str, False),
        'ephemeral_size': (integer, False),
        'image_id': (str, False),
        'snapshot_id': (str, False),
        'swap_size': (integer, False),
        'volume_id': (str, False),
        'volume_size': (integer, False),
    }

    def validate(self):
        if 'device_type' in self.resource:
            device_type = self.resource['device_type']
            if device_type not in ['cdrom', 'disk']:
                raise ValueError(
                    "The device_type attribute "
                    "must be either cdrom or disk")

        if 'disk_bus' in self.resource:
            disk_bus = self.resource['disk_bus']
            if disk_bus not in ['ide', 'lame_bus', 'scsi', 'usb', 'virtio']:
                raise ValueError(
                    "The device_bus attribute "
                    "must be one of ide, lame_bus, scsi, usb or virtio")

        if 'ephemeral_format' in self.resource:
            ephemeral_format = self.resource['ephemeral_format']
            if ephemeral_format not in ['ext2', 'ext3', 'ext4', 'xfs', 'ntfs']:
                raise ValueError(
                    "The device_type attribute "
                    "must be one of ext2, ext3, ext4, xfs, ntfs")


class Network(AWSProperty):
    props = {
        'fixed_ip': (str, False),
        'network': (str, False),
        'port': (network_port, False),
    }


class FloatingIP(AWSObject):
    resource_type = "OS::Nova::FloatingIP"

    props = {
        'pool': (str, False),
    }


class FloatingIPAssociation(AWSObject):
    resource_type = "OS::Nova::FloatingIPAssociation"

    props = {
        'floating_ip': (str, True),
        'server_ip': (str, True),
    }


class KeyPair(AWSObject):
    resource_type = "OS::Nova::KeyPair"

    props = {
        'name': (str, True),
        'public_key': (str, False),
        'save_private_key': (boolean, False),
    }


class Server(AWSObject):
    resource_type = "OS::Nova::Server"

    props = {
        'admin_pass': (str, False),
        'admin_user': (str, False),
        'availability_zone': (str, False),
        'block_device_mapping': (list, False),
        'block_device_mapping_v2': (list, False),
        'config_drive': (str, False),
        'diskConfig': (str, False),
        'flavor': (str, False),
        'flavor_update_policy': (str, False),
        'image': (str, True),
        'image_update_policy': (str, False),
        'key_name': (str, False),
        'metadata': (dict, False),
        'name': (str, False),
        'personality': (dict, False),
        'networks': (list, True),
        'reservation_id': (str, False),
        'scheduler_hints': (dict, False),
        'security_groups': (list, False),
        'software_config_transport': (str, False),
        'user_data': (str, False),
        'user_data_format': (str, False),
    }

    def validate(self):
        if 'diskConfig' in self.resource:
            diskConfig = self.resource['diskConfig']
            if diskConfig not in ['AUTO', 'MANUAL']:
                raise ValueError(
                    "The diskConfig attribute "
                    "must be either AUTO or MANUAL")

        if 'flavor_update_policy' in self.resource:
            flavor_update_policy = self.resource['flavor_update_policy']
            if flavor_update_policy not in ['RESIZE', 'REPLACE']:
                raise ValueError(
                    "The flavor_update_policy attribute "
                    "must be either RESIZE or REPLACE")

        if 'image_update_policy' in self.resource:
            image_update_policy = self.resource['flavor_update_policy']
            if image_update_policy not in ['REBUILD', 'REPLACE',
                                           'REBUILD_PRESERVE_EPHEMERAL']:
                raise ValueError(
                    "The image_update_policy attribute "
                    "must be either REBUILD, REPLACE or "
                    "REBUILD_PRESERVE_EPHEMERAL")

        if 'software_config_transport' in self.resource:
            sct = self.resource['software_config_transport']
            if sct not in ['POLL_SERVER_CFN', 'POLL_SERVER_HEAT']:
                raise ValueError(
                    "The software_config_transport attribute "
                    "must be either POLL_SERVER_CFN or POLL_SERVER_HEAT")

        if 'user_data_format' in self.resource:
            user_data_format = self.resource['user_data_format']
            if user_data_format not in ['HEAT_CFNTOOLS', 'RAW']:
                raise ValueError(
                    "The user_data_format attribute "
                    "must be either HEAT_CFNTOOLS or RAW")

        return True



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject
from . import AWSProperty
from .validators import integer


class AccelerationSettings(AWSProperty):
    props = {
        'Mode': (str, True),
    }


class JobTemplate(AWSObject):
    resource_type = "AWS::MediaConvert::JobTemplate"

    props = {
        'AccelerationSettings': (AccelerationSettings, False),
        'Category': (str, False),
        'Description': (str, False),
        'Name': (str, False),
        'Priority': (integer, False),
        'Queue': (str, False),
        'SettingsJson': (dict, True),
        'StatusUpdateInterval': (str, False),
        'Tags': (dict, False),
    }


class Preset(AWSObject):
    resource_type = "AWS::MediaConvert::Preset"

    props = {
        'Category': (str, False),
        'Description': (str, False),
        'Name': (str, False),
        'SettingsJson': (dict, True),
        'Tags': (dict, False),
    }


class Queue(AWSObject):
    resource_type = "AWS::MediaConvert::Queue"

    props = {
        'Description': (str, False),
        'Name': (str, False),
        'PricingPlan': (str, False),
        'Status': (str, False),
        'Tags': (dict, False),
    }



# Copyright (c) 2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class LustreConfiguration(AWSProperty):
    props = {
        'ExportPath': (str, False),
        'ImportedFileChunkSize': (integer, False),
        'ImportPath': (str, False),
        'WeeklyMaintenanceStartTime': (str, False),

    }


class SelfManagedActiveDirectoryConfiguration(AWSProperty):
    props = {
        'DnsIps': ([str], False),
        'DomainName': (str, False),
        'FileSystemAdministratorsGroup': (str, False),
        'OrganizationalUnitDistinguishedName': (str, False),
        'Password': (str, False),
        'UserName': (str, False),
    }


class WindowsConfiguration(AWSProperty):
    props = {
        'ActiveDirectoryId': (str, False),
        'AutomaticBackupRetentionDays': (integer, False),
        'CopyTagsToBackups': (boolean, False),
        'DailyAutomaticBackupStartTime': (str, False),
        'DeploymentType': (str, False),
        'PreferredSubnetId': (str, False),
        'SelfManagedActiveDirectoryConfiguration':
            (SelfManagedActiveDirectoryConfiguration, False),
        'ThroughputCapacity': (integer, False),
        'WeeklyMaintenanceStartTime': (str, False),
    }


class FileSystem(AWSObject):
    resource_type = 'AWS::FSx::FileSystem'

    props = {
        'BackupId': (str, False),
        'FileSystemType': (str, False),
        'KmsKeyId': (str, False),
        'LustreConfiguration': (LustreConfiguration, False),
        'SecurityGroupIds': ([str], False),
        'StorageCapacity': (integer, False),
        'SubnetIds': ([str], False),
        'Tags': (Tags, False),
        'WindowsConfiguration': (WindowsConfiguration, False),
    }



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSProperty, AWSObject, Tags
from .validators import json_checker, boolean


class IEMap(AWSProperty):
    props = {
        'ACCOUNT': ([str], False),
    }


class Policy(AWSObject):
    resource_type = "AWS::FMS::Policy"

    props = {
        'DeleteAllPolicyResources': (boolean, False),
        'ExcludeMap': (IEMap, False),
        'ExcludeResourceTags': (boolean, True),
        'IncludeMap': (IEMap, False),
        'PolicyName': (str, True),
        'RemediationEnabled': (boolean, True),
        'ResourceTags': (Tags, False),
        'ResourceType': (str, True),
        'ResourceTypeList': ([str], True),
        'SecurityServicePolicyData': (json_checker, True),
        'Tags': (Tags, False),
    }


class NotificationChannel(AWSObject):
    resource_type = "AWS::FMS::NotificationChannel"

    props = {
        'SnsRoleName': (str, True),
        'SnsTopicArn': (str, True),
    }



from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean, double, integer_range, json_checker, positive_integer
)


def validate_authorizer_ttl(ttl_value):
    """ Validate authorizer ttl timeout
    :param ttl_value: The TTL timeout in seconds
    :return: The provided TTL value if valid
    """
    ttl_value = int(positive_integer(ttl_value))
    if ttl_value > 3600:
        raise ValueError("The AuthorizerResultTtlInSeconds should be <= 3600")
    return ttl_value


class AccessLogSetting(AWSProperty):

    props = {
        "DestinationArn": (str, False),
        "Format": (str, False)
    }


class Account(AWSObject):
    resource_type = "AWS::ApiGateway::Account"

    props = {
        "CloudWatchRoleArn": (str, False)
    }


class StageKey(AWSProperty):

    props = {
        "RestApiId": (str, False),
        "StageName": (str, False)
    }


class ApiKey(AWSObject):
    resource_type = "AWS::ApiGateway::ApiKey"

    props = {
        "CustomerId": (str, False),
        "Description": (str, False),
        "Enabled": (boolean, False),
        "GenerateDistinctId": (boolean, False),
        "Name": (str, False),
        "StageKeys": ([StageKey], False),
        "Tags": (Tags, False),
        "Value": (str, False)
    }


class Authorizer(AWSObject):
    resource_type = "AWS::ApiGateway::Authorizer"

    props = {
        "AuthType": (str, False),
        "AuthorizerCredentials": (str, False),
        "AuthorizerResultTtlInSeconds": (validate_authorizer_ttl, False),
        "AuthorizerUri": (str, True),
        "IdentitySource": (str, True),
        "IdentityValidationExpression": (str, False),
        "Name": (str, True),
        "ProviderARNs": ([str], False),
        "RestApiId": (str, False),
        "Type": (str, True)
    }


class BasePathMapping(AWSObject):
    resource_type = "AWS::ApiGateway::BasePathMapping"

    props = {
        "BasePath": (str, False),
        "DomainName": (str, True),
        "RestApiId": (str, True),
        "Stage": (str, False)
    }


# Represents:
# https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-stage-canarysetting.html
class CanarySetting(AWSProperty):

    props = {
        "DeploymentId": (str, False),
        "PercentTraffic": ([double], False),
        "StageVariableOverrides": (dict, False),
        "UseStageCache": (boolean, False),
    }


StageCanarySetting = CanarySetting


class ClientCertificate(AWSObject):
    resource_type = "AWS::ApiGateway::ClientCertificate"

    props = {
        "Description": (str, False),
        "Tags": (Tags, False),
    }


# Represents:
# http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-canarysetting.html
# and
# https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-deploymentcanarysettings.html
class DeploymentCanarySettings(AWSProperty):

    props = {
        "PercentTraffic": ([double], False),
        "StageVariableOverrides": (dict, False),
        "UseStageCache": (boolean, False),
    }


DeploymentCanarySetting = DeploymentCanarySettings


class MethodSetting(AWSProperty):

    props = {
        "CacheDataEncrypted": (bool, False),
        "CacheTtlInSeconds": (positive_integer, False),
        "CachingEnabled": (bool, False),
        "DataTraceEnabled": (bool, False),
        "HttpMethod": (str, True),
        "LoggingLevel": (str, False),
        "MetricsEnabled": (bool, False),
        "ResourcePath": (str, True),
        "ThrottlingBurstLimit": (positive_integer, False),
        "ThrottlingRateLimit": (positive_integer, False)
    }


class StageDescription(AWSProperty):

    props = {
        "AccessLogSetting": (AccessLogSetting, False),
        "CacheClusterEnabled": (bool, False),
        "CacheClusterSize": (str, False),
        "CacheDataEncrypted": (bool, False),
        "CacheTtlInSeconds": (positive_integer, False),
        "CachingEnabled": (bool, False),
        "CanarySetting": (DeploymentCanarySettings, False),
        "ClientCertificateId": (str, False),
        "DataTraceEnabled": (bool, False),
        "Description": (str, False),
        "LoggingLevel": (str, False),
        "MethodSettings": ([MethodSetting], False),
        "MetricsEnabled": (bool, False),
        "StageName": (str, False),
        "Tags": ((Tags, list), False),
        "ThrottlingBurstLimit": (positive_integer, False),
        "ThrottlingRateLimit": (positive_integer, False),
        "Variables": (dict, False),
    }

    def validate(self):
        if 'StageName' in self.properties:
            raise DeprecationWarning(
                "The StageName property has been deprecated "
                "in StageDescription"
            )


class Deployment(AWSObject):
    resource_type = "AWS::ApiGateway::Deployment"

    props = {
        "DeploymentCanarySettings": (DeploymentCanarySettings, False),
        "Description": (str, False),
        "RestApiId": (str, True),
        "StageDescription": (StageDescription, False),
        "StageName": (str, False)
    }


class Location(AWSProperty):
    props = {
        "Method": (str, False),
        "Name": (str, False),
        "Path": (str, False),
        "StatusCode": (str, False),
        "Type": (str, False),
    }


class DocumentationPart(AWSObject):
    resource_type = "AWS::ApiGateway::DocumentationPart"

    props = {
        "Location": (Location, True),
        "Properties": (str, True),
        "RestApiId": (str, True),
    }


class DocumentationVersion(AWSObject):
    resource_type = "AWS::ApiGateway::DocumentationVersion"

    props = {
        "Description": (str, False),
        "DocumentationVersion": (str, True),
        "RestApiId": (str, True),
    }


class EndpointConfiguration(AWSProperty):

    props = {
        "Types": ([str], False),
        "VpcEndpointIds": ([str], False),
    }


class DomainName(AWSObject):
    resource_type = "AWS::ApiGateway::DomainName"

    props = {
        "CertificateArn": (str, False),
        "DomainName": (str, True),
        "EndpointConfiguration": (EndpointConfiguration, False),
        "RegionalCertificateArn": (str, False),
        "SecurityPolicy": (str, False),
        "Tags": (Tags, False),
    }


class IntegrationResponse(AWSProperty):

    props = {
        "ContentHandling": (str, False),
        "ResponseParameters": (dict, False),
        "ResponseTemplates": (dict, False),
        "SelectionPattern": (str, False),
        "StatusCode": (str, False)
    }


class Integration(AWSProperty):

    props = {
        "CacheKeyParameters": ([str], False),
        "CacheNamespace": (str, False),
        "ConnectionId": (str, False),
        "ConnectionType": (str, False),
        "ContentHandling": (str, False),
        "Credentials": (str, False),
        "IntegrationHttpMethod": (str, False),
        "IntegrationResponses": ([IntegrationResponse], False),
        "PassthroughBehavior": (str, False),
        "RequestParameters": (dict, False),
        "RequestTemplates": (dict, False),
        "TimeoutInMillis": (integer_range(50, 29000), False),
        "Type": (str, True),
        "Uri": (str, False)
    }


class MethodResponse(AWSProperty):

    props = {
        "ResponseModels": (dict, False),
        "ResponseParameters": (dict, False),
        "StatusCode": (str, True)
    }


class Method(AWSObject):
    resource_type = "AWS::ApiGateway::Method"

    props = {
        "ApiKeyRequired": (bool, False),
        "AuthorizationScopes": ([str], False),
        "AuthorizationType": (str, True),
        "AuthorizerId": (str, False),
        "HttpMethod": (str, True),
        "Integration": (Integration, False),
        "MethodResponses": ([MethodResponse], False),
        "OperationName": (str, False),
        "RequestModels": (dict, False),
        "RequestParameters": (dict, False),
        "RequestValidatorId": (str, False),
        "ResourceId": (str, True),
        "RestApiId": (str, True)
    }


class Model(AWSObject):
    resource_type = "AWS::ApiGateway::Model"

    props = {
        "ContentType": (str, False),
        "Description": (str, False),
        "Name": (str, False),
        "RestApiId": (str, True),
        "Schema": ((str, dict), False)
    }

    def validate(self):
        name = 'Schema'
        if name in self.properties:
            schema = self.properties.get(name)
            self.properties[name] = json_checker(schema)


class RequestValidator(AWSObject):
    resource_type = "AWS::ApiGateway::RequestValidator"

    props = {
        "Name": (str, True),
        "RestApiId": (str, True),
        "ValidateRequestBody": (boolean, False),
        "ValidateRequestParameters": (boolean, False),
    }


class Resource(AWSObject):
    resource_type = "AWS::ApiGateway::Resource"

    props = {
        "ParentId": (str, True),
        "PathPart": (str, True),
        "RestApiId": (str, True)
    }


class S3Location(AWSProperty):

    props = {
        "Bucket": (str, False),
        "ETag": (str, False),
        "Key": (str, False),
        "Version": (str, False)
    }


class RestApi(AWSObject):
    resource_type = "AWS::ApiGateway::RestApi"

    props = {
        "ApiKeySourceType": (str, False),
        "BinaryMediaTypes": ([str], False),
        "Body": (dict, False),
        "BodyS3Location": (S3Location, False),
        "CloneFrom": (str, False),
        "Description": (str, False),
        "EndpointConfiguration": (EndpointConfiguration, False),
        "FailOnWarnings": (str, False),
        "MinimumCompressionSize": (positive_integer, False),
        "Name": (str, False),
        "Parameters": (dict, False),
        "Policy": (dict, False),
        "Tags": (Tags, False),
    }


class Stage(AWSObject):
    resource_type = "AWS::ApiGateway::Stage"

    props = {
        "AccessLogSetting": (AccessLogSetting, False),
        "CacheClusterEnabled": (bool, False),
        "CacheClusterSize": (str, False),
        "CanarySetting": (StageCanarySetting, False),
        "ClientCertificateId": (str, False),
        "DeploymentId": (str, True),
        "Description": (str, False),
        "DocumentationVersion": (str, False),
        "MethodSettings": ([MethodSetting], False),
        "RestApiId": (str, True),
        "StageName": (str, True),
        "Tags": ((Tags, list), False),
        "TracingEnabled": (bool, False),
        "Variables": (dict, False),
    }


class QuotaSettings(AWSProperty):
    props = {
        "Limit": (positive_integer, False),
        "Offset": (positive_integer, False),
        "Period": (str, False),
    }


class ThrottleSettings(AWSProperty):
    props = {
        "BurstLimit": (positive_integer, False),
        "RateLimit": (positive_integer, False),
    }


class ApiStage(AWSProperty):
    props = {
        "ApiId": (str, False),
        "Stage": (str, False),
        "Throttle": (ThrottleSettings, False),
    }


class UsagePlan(AWSObject):
    resource_type = "AWS::ApiGateway::UsagePlan"

    props = {
        "ApiStages": ([ApiStage], False),
        "Description": (str, False),
        "Quota": (QuotaSettings, False),
        "Tags": (Tags, False),
        "Throttle": (ThrottleSettings, False),
        "UsagePlanName": (str, False),
    }


class UsagePlanKey(AWSObject):
    resource_type = "AWS::ApiGateway::UsagePlanKey"

    props = {
        "KeyId": (str, True),
        "KeyType": (str, True),
        "UsagePlanId": (str, True),
    }


def validate_gateway_response_type(response_type):
    """ Validate response type
    :param response_type: The GatewayResponse response type
    :return: The provided value if valid
    """
    valid_response_types = [
        "ACCESS_DENIED",
        "API_CONFIGURATION_ERROR",
        "AUTHORIZER_FAILURE",
        "AUTHORIZER_CONFIGURATION_ERROR",
        "BAD_REQUEST_PARAMETERS",
        "BAD_REQUEST_BODY",
        "DEFAULT_4XX",
        "DEFAULT_5XX",
        "EXPIRED_TOKEN",
        "INVALID_SIGNATURE",
        "INTEGRATION_FAILURE",
        "INTEGRATION_TIMEOUT",
        "INVALID_API_KEY",
        "MISSING_AUTHENTICATION_TOKEN",
        "QUOTA_EXCEEDED",
        "REQUEST_TOO_LARGE",
        "RESOURCE_NOT_FOUND",
        "THROTTLED",
        "UNAUTHORIZED",
        "UNSUPPORTED_MEDIA_TYPE"
    ]
    if response_type not in valid_response_types:
        raise ValueError(
            "{} is not a valid ResponseType".format(response_type)
        )
    return response_type


class GatewayResponse(AWSObject):
    resource_type = "AWS::ApiGateway::GatewayResponse"

    props = {
        "ResponseParameters": (dict, False),
        "ResponseTemplates": (dict, False),
        "ResponseType": (validate_gateway_response_type, True),
        "RestApiId": (str, True),
        "StatusCode": (str, False)
    }


class VpcLink(AWSObject):
    resource_type = "AWS::ApiGateway::VpcLink"

    props = {
        'Description': (str, False),
        'Name': (str, True),
        'TargetArns': ([str], True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import integer, positive_integer, network_port, boolean


VALID_RULETYPES = ('SYSTEM', 'FORWARD')


def validate_ruletype(ruletype):
    """Validate RuleType for ResolverRule."""

    if ruletype not in VALID_RULETYPES:
        raise ValueError("Rule type must be one of: %s" %
                         ", ".join(VALID_RULETYPES))
    return ruletype


class AliasTarget(AWSProperty):
    props = {
        'HostedZoneId': (str, True),
        'DNSName': (str, True),
        'EvaluateTargetHealth': (boolean, False)
    }

    def __init__(self,
                 hostedzoneid=None,
                 dnsname=None,
                 evaluatetargethealth=None,
                 **kwargs):
        # provided for backward compatibility
        if hostedzoneid is not None:
            kwargs['HostedZoneId'] = hostedzoneid
        if dnsname is not None:
            kwargs['DNSName'] = dnsname
        if evaluatetargethealth is not None:
            kwargs['EvaluateTargetHealth'] = evaluatetargethealth
        super(AliasTarget, self).__init__(**kwargs)


class GeoLocation(AWSProperty):
    props = {
        'ContinentCode': (str, False),
        'CountryCode': (str, False),
        'SubdivisionCode': (str, False),
    }


class BaseRecordSet(object):
    props = {
        'AliasTarget': (AliasTarget, False),
        'Comment': (str, False),
        'Failover': (str, False),
        'GeoLocation': (GeoLocation, False),
        'HealthCheckId': (str, False),
        'HostedZoneId': (str, False),
        'HostedZoneName': (str, False),
        'MultiValueAnswer': (boolean, False),
        'Name': (str, True),
        'Region': (str, False),
        'ResourceRecords': (list, False),
        'SetIdentifier': (str, False),
        'TTL': (integer, False),
        'Type': (str, True),
        'Weight': (integer, False),
    }


class RecordSetType(AWSObject, BaseRecordSet):
    # This is a top-level resource
    resource_type = "AWS::Route53::RecordSet"


class RecordSet(AWSProperty, BaseRecordSet):
    # This is for use in a list with RecordSetGroup (below)
    pass


class RecordSetGroup(AWSObject):
    resource_type = "AWS::Route53::RecordSetGroup"

    props = {
        'HostedZoneId': (str, False),
        'HostedZoneName': (str, False),
        'RecordSets': (list, False),
        'Comment': (str, False),
    }


class AlarmIdentifier(AWSProperty):
    props = {
        'Name': (str, True),
        'Region': (str, True),
    }


class HealthCheckConfiguration(AWSProperty):
    props = {
        'AlarmIdentifier': (AlarmIdentifier, False),
        'ChildHealthChecks': ([str], False),
        'EnableSNI': (boolean, False),
        'FailureThreshold': (positive_integer, False),
        'FullyQualifiedDomainName': (str, False),
        'HealthThreshold': (positive_integer, False),
        'InsufficientDataHealthStatus': (str, False),
        'Inverted': (boolean, False),
        'IPAddress': (str, False),
        'MeasureLatency': (boolean, False),
        'Port': (network_port, False),
        'Regions': ([str], False),
        'RequestInterval': (positive_integer, False),
        'ResourcePath': (str, False),
        'SearchString': (str, False),
        'Type': (str, True),
    }


class HealthCheck(AWSObject):
    resource_type = "AWS::Route53::HealthCheck"

    props = {
        'HealthCheckConfig': (HealthCheckConfiguration, True),
        'HealthCheckTags': (Tags, False),
    }


class HostedZoneConfiguration(AWSProperty):
    props = {
        'Comment': (str, False),
    }


class HostedZoneVPCs(AWSProperty):
    props = {
        'VPCId': (str, True),
        'VPCRegion': (str, True),
    }


class QueryLoggingConfig(AWSProperty):
    props = {
        'CloudWatchLogsLogGroupArn': (str, True),
    }


class HostedZone(AWSObject):
    resource_type = "AWS::Route53::HostedZone"

    props = {
        'HostedZoneConfig': (HostedZoneConfiguration, False),
        'HostedZoneTags': (Tags, False),
        'Name': (str, True),
        'QueryLoggingConfig': (QueryLoggingConfig, False),
        'VPCs': ([HostedZoneVPCs], False),
    }


class IpAddressRequest(AWSProperty):
    props = {
        'Ip': (str, False),
        'SubnetId': (str, True),
    }


class ResolverEndpoint(AWSObject):
    resource_type = "AWS::Route53Resolver::ResolverEndpoint"

    props = {
        'Direction': (str, True),
        'IpAddresses': ([IpAddressRequest], True),
        'Name': (str, False),
        'SecurityGroupIds': ([str], True),
        'Tags': (Tags, False),
    }


class TargetAddress(AWSProperty):
    props = {
        'Ip': (str, True),
        'Port': (str, True),
    }


class ResolverRule(AWSObject):
    resource_type = "AWS::Route53Resolver::ResolverRule"

    props = {
        'DomainName': (str, True),
        'Name': (str, False),
        'ResolverEndpointId': (str, False),
        'RuleType': (validate_ruletype, True),
        'Tags': (Tags, False),
        'TargetIps': ([TargetAddress], False),
    }


class ResolverRuleAssociation(AWSObject):
    resource_type = "AWS::Route53Resolver::ResolverRuleAssociation"

    props = {
        'Name': (str, False),
        'ResolverRuleId': (str, True),
        'VPCId': (str, True),
    }
